@@ -10,7 +10,6 @@ import structlog
 import uuid
 from alembic import command  # type: ignore
 from pydantic import Field
-from sqlalchemy.engine import Engine, create_engine
 from typing import (
     TYPE_CHECKING,
     Any,
@@ -23,23 +22,19 @@ from typing import (
     Union,
 )
 
-from kiara.context.config import KiaraContextConfig, KiaraGlobalConfig
+from kiara.context.config import KiaraConfig, KiaraContextConfig
 from kiara.data_types import DataType
-from kiara.defaults import (
-    CONTEXT_INFO_CATEGORY_ID,
-    KIARA_DB_MIGRATIONS_CONFIG,
-    KIARA_DB_MIGRATIONS_FOLDER,
-)
+from kiara.defaults import KIARA_DB_MIGRATIONS_CONFIG, KIARA_DB_MIGRATIONS_FOLDER
 from kiara.interfaces import get_console
+from kiara.interfaces.python_api import StoreValueResult, StoreValuesResult
 from kiara.models import KiaraModel
-from kiara.models.info import InfoModelGroup, KiaraInfoModel
+from kiara.models.info import InfoModelGroup, ItemInfo, KiaraModelClassesInfo
 from kiara.models.module import KiaraModuleTypeInfo, ModuleTypeClassesInfo
 from kiara.models.module.manifest import Manifest
 from kiara.models.module.operation import OperationGroupInfo, OperationTypeClassesInfo
 from kiara.models.runtime_environment import RuntimeEnvironment
 from kiara.models.values.data_type import DataTypeClassesInfo
-from kiara.models.values.value import Value
-from kiara.models.values.value_metadata import MetadataTypeClassesInfo
+from kiara.models.values.value import ValueMap
 from kiara.registries import KiaraArchive
 from kiara.registries.aliases import AliasRegistry
 from kiara.registries.data import DataRegistry
@@ -49,13 +44,12 @@ from kiara.registries.events.metadata import CreateMetadataDestinies
 from kiara.registries.events.registry import EventRegistry
 from kiara.registries.ids import ID_REGISTRY
 from kiara.registries.jobs import JobRegistry
+from kiara.registries.models import ModelRegistry
 from kiara.registries.modules import ModuleRegistry
 from kiara.registries.operations import OperationRegistry
 from kiara.registries.types import TypeRegistry
 from kiara.utils import is_debug, log_message
 from kiara.utils.class_loading import find_all_archive_types
-from kiara.utils.db import orm_json_deserialize, orm_json_serialize
-from kiara.utils.metadata import find_metadata_models
 from kiara.utils.operations import filter_operations
 
 if TYPE_CHECKING:
@@ -113,8 +107,8 @@ class Kiara(object):
     def __init__(self, config: Optional[KiaraContextConfig] = None):
 
         if not config:
-            kc = KiaraGlobalConfig()
-            config = kc.get_context()
+            kc = KiaraConfig()
+            config = kc.get_context_config()
 
         self._id: uuid.UUID = ID_REGISTRY.generate(
             id=uuid.UUID(config.context_id), obj=self
@@ -122,17 +116,17 @@ class Kiara(object):
         ID_REGISTRY.update_metadata(self._id, kiara_id=self._id)
         self._config: KiaraContextConfig = config
 
-        if is_debug():
-            echo = True
-        else:
-            echo = False
-        self._engine: Engine = create_engine(
-            self._config.db_url,
-            echo=echo,
-            future=True,
-            json_serializer=orm_json_serialize,
-            json_deserializer=orm_json_deserialize,
-        )
+        # if is_debug():
+        #     echo = True
+        # else:
+        #     echo = False
+        # self._engine: Engine = create_engine(
+        #     self._config.db_url,
+        #     echo=echo,
+        #     future=True,
+        #     json_serializer=orm_json_serialize,
+        #     json_deserializer=orm_json_deserialize,
+        # )
 
         # self._run_alembic_migrations()
         # self._envs: Optional[Mapping[str, EnvironmentOrm]] = None
@@ -144,6 +138,8 @@ class Kiara(object):
         self._module_registry: ModuleRegistry = ModuleRegistry()
         self._operation_registry: OperationRegistry = OperationRegistry(kiara=self)
 
+        self._kiara_model_registry: ModelRegistry = ModelRegistry.instance()
+
         self._alias_registry: AliasRegistry = AliasRegistry(kiara=self)
         self._destiny_registry: DestinyRegistry = DestinyRegistry(kiara=self)
 
@@ -226,6 +222,10 @@ class Kiara(object):
     def module_registry(self) -> ModuleRegistry:
         return self._module_registry
 
+    @property
+    def kiara_model_registry(self) -> ModelRegistry:
+        return self._kiara_model_registry
+
     @property
     def alias_registry(self) -> AliasRegistry:
         return self._alias_registry
@@ -240,7 +240,8 @@ class Kiara(object):
 
     @property
     def operation_registry(self) -> OperationRegistry:
-        return self._operation_registry
+        op_registry = self._operation_registry
+        return op_registry
 
     @property
     def data_registry(self) -> DataRegistry:
@@ -276,7 +277,38 @@ class Kiara(object):
     # ===================================================================================================
     # kiara session API methods
 
-    def create_module(self, manifest: Manifest) -> "KiaraModule":
+    def create_manifest(
+        self, module_or_operation: str, config: Optional[Mapping[str, Any]] = None
+    ) -> Manifest:
+
+        if config is None:
+            config = {}
+
+        if module_or_operation in self.module_type_names:
+
+            manifest: Manifest = Manifest(
+                module_type=module_or_operation, module_config=config
+            )
+
+        elif module_or_operation in self.operation_registry.operation_ids:
+
+            if config:
+                raise Exception(
+                    f"Specified run target '{module_or_operation}' is an operation, additional module configuration is not allowed (yet)."
+                )
+            manifest = self.operation_registry.get_operation(module_or_operation)
+
+        elif os.path.isfile(module_or_operation):
+            raise NotImplementedError()
+
+        else:
+            raise Exception(
+                f"Can't assemble operation, invalid operation/module name: {module_or_operation}. Must be registered module or operation name, or file."
+            )
+
+        return manifest
+
+    def create_module(self, manifest: Union[Manifest, str]) -> "KiaraModule":
         """Create a [KiaraModule][kiara.module.KiaraModule] object from a module configuration.
 
         Arguments:
@@ -285,8 +317,66 @@ class Kiara(object):
 
         return self._module_registry.create_module(manifest=manifest)
 
-    def get_value(self, value: Union[uuid.UUID, str, Value]):
-        pass
+    def queue(
+        self, manifest: Manifest, inputs: Mapping[str, Any], wait: bool = False
+    ) -> uuid.UUID:
+        """Queue a job with the specified manifest and inputs.
+
+        Arguments:
+           manifest: the job manifest
+           inputs: the job inputs
+           wait: whether to wait for the job to be finished before returning
+
+        Returns:
+            the job id that can be used to look up job status & results
+        """
+
+        return self.job_registry.execute(manifest=manifest, inputs=inputs, wait=wait)
+
+    def process(self, manifest: Manifest, inputs: Mapping[str, Any]) -> ValueMap:
+        """Queue a job with the specified manifest and inputs.
+
+        Arguments:
+           manifest: the job manifest
+           inputs: the job inputs
+           wait: whether to wait for the job to be finished before returning
+
+        Returns
+        """
+
+        return self.job_registry.execute_and_retrieve(manifest=manifest, inputs=inputs)
+
+    def save_values(
+        self, values: ValueMap, alias_map: Mapping[str, Iterable[str]]
+    ) -> StoreValuesResult:
+
+        _values = {}
+        for field_name in values.field_names:
+            value = values.get_value_obj(field_name)
+            _values[field_name] = value
+            self.data_registry.store_value(value=value)
+        stored = {}
+        for field_name, field_aliases in alias_map.items():
+
+            value = _values[field_name]
+            try:
+                if field_aliases:
+                    self.alias_registry.register_aliases(value.value_id, *field_aliases)
+
+                stored[field_name] = StoreValueResult.construct(
+                    value=value, aliases=sorted(field_aliases), error=None
+                )
+
+            except Exception as e:
+                if is_debug():
+                    import traceback
+
+                    traceback.print_exc()
+                stored[field_name] = StoreValueResult.construct(
+                    value=value, aliases=sorted(field_aliases), error=str(e)
+                )
+
+        return StoreValuesResult.construct(__root__=stored)
 
     # def run(self, module_or_operation: str, module_config: Mapping[str, Any] = None):
     #
@@ -343,14 +433,23 @@ class KiaraContextInfo(KiaraModel):
             kiara=kiara, pkg_name=package_filter, **kiara.operation_registry.operations
         )
 
-        metadata_types = find_metadata_models(only_for_package=package_filter)
+        model_registry = kiara.kiara_model_registry
+        if package_filter:
+            kiara_models = model_registry.get_models_for_package(
+                package_name=package_filter
+            )
+        else:
+            kiara_models = model_registry.all_models
+
+        # metadata_types = find_metadata_models(only_for_package=package_filter)
 
         return KiaraContextInfo.construct(
             kiara_id=kiara.id,
             package_filter=package_filter,
             data_types=data_types,
             module_types=modules,
-            metadata_types=metadata_types,
+            kiara_model_types=kiara_models,
+            # metadata_types=metadata_types,
             operation_types=operation_types,
             operations=operations,
         )
@@ -363,9 +462,12 @@ class KiaraContextInfo(KiaraModel):
     module_types: ModuleTypeClassesInfo = Field(
         description="The included kiara module types."
     )
-    metadata_types: MetadataTypeClassesInfo = Field(
-        description="The included value metadata types."
+    kiara_model_types: KiaraModelClassesInfo = Field(
+        description="The included model classes."
     )
+    # metadata_types: MetadataTypeClassesInfo = Field(
+    #     description="The included value metadata types."
+    # )
     operation_types: OperationTypeClassesInfo = Field(
         description="The included operation types."
     )
@@ -377,29 +479,28 @@ class KiaraContextInfo(KiaraModel):
         else:
             return f"{self.kiara_id}.package_{self.package_filter}"
 
-    def _retrieve_category_id(self) -> str:
-        return CONTEXT_INFO_CATEGORY_ID
-
     def _retrieve_data_to_hash(self) -> Any:
         return {"kiara_id": self.kiara_id, "package": self.package_filter}
 
-    def get_info(self, item_type: str, item_id: str) -> KiaraInfoModel:
+    def get_info(self, item_type: str, item_id: str) -> ItemInfo:
 
         if "data_type" == item_type or "data_types" == item_type:
             group_info: InfoModelGroup = self.data_types
         elif "module" in item_type:
             group_info = self.module_types
-        elif "metadata" in item_type:
-            group_info = self.metadata_types
+        # elif "metadata" in item_type:
+        #     group_info = self.metadata_types
         elif "operation_type" in item_type or "operation_types" in item_type:
             group_info = self.operation_types
         elif "operation" in item_type:
             group_info = self.operations
+        elif "kiara_model" in item_type:
+            group_info = self.kiara_model_types
         else:
             item_types = [
                 "data_type",
                 "module_type",
-                "metadata_type",
+                "kiara_model_type",
                 "operation_type",
                 "operation",
             ]
@@ -415,8 +516,10 @@ class KiaraContextInfo(KiaraModel):
             result["data_types"] = self.data_types
         if self.module_types or not skip_empty_types:
             result["module_types"] = self.module_types
-        if self.metadata_types or not skip_empty_types:
-            result["metadata_types"] = self.metadata_types
+        if self.kiara_model_types or not skip_empty_types:
+            result["kiara_model_types"] = self.kiara_model_types
+        # if self.metadata_types or not skip_empty_types:
+        #     result["metadata_types"] = self.metadata_types
         if self.operation_types or not skip_empty_types:
             result["operation_types"] = self.operation_types
         if self.operations or not skip_empty_types:
