@@ -25,12 +25,16 @@ else:
 KIARA_RESOURCES_FOLDER = os.path.join(KIARA_MODULE_BASE_FOLDER, "resources")
 """Default resources folder for this package."""
 
-KIARA_MAIN_CONFIG_FILE = os.path.join(kiara_app_dirs.user_config_dir, "config.yaml")
+KIARA_CONFIG_FILE_NAME = "kiara.config"
+KIARA_MAIN_CONFIG_FILE = os.path.join(
+    kiara_app_dirs.user_config_dir, KIARA_CONFIG_FILE_NAME
+)
+KIARA_MAIN_CONTEXTS_PATH = os.path.join(kiara_app_dirs.user_config_dir, "contexts")
 
-KIARA_CONTEXTS_FOLDER = os.path.join(kiara_app_dirs.user_data_dir, "contexts")
-KIARA_STORES_FOLDER = os.path.join(kiara_app_dirs.user_data_dir, "stores")
+# KIARA_CONTEXTS_FOLDER = os.path.join(kiara_app_dirs.user_data_dir, "contexts")
+# KIARA_STORES_FOLDER = os.path.join(kiara_app_dirs.user_data_dir, "stores")
 
-KIARA_DB_FILE = os.path.join(kiara_app_dirs.user_data_dir, "kiara.db")
+# KIARA_DB_FILE = os.path.join(kiara_app_dirs.user_data_dir, "kiara.db")
 KIARA_DB_MIGRATIONS_CONFIG = os.path.join(
     KIARA_RESOURCES_FOLDER, "database", "alembic.ini"
 )
@@ -38,12 +42,6 @@ KIARA_DB_MIGRATIONS_FOLDER = os.path.join(KIARA_RESOURCES_FOLDER, "database", "k
 
 USER_PIPELINES_FOLDER = os.path.join(kiara_app_dirs.user_config_dir, "pipelines")
 
-KIARA_DATA_STORE_DIR = os.path.join(
-    kiara_app_dirs.user_data_dir, "data-store", "default"
-)
-KIARA_DATA_DIR = os.path.join(KIARA_DATA_STORE_DIR, "data")
-KIARA_METADATA_DIR = os.path.join(KIARA_DATA_STORE_DIR, "metadata")
-KIARA_ALIASES_DIR = os.path.join(KIARA_DATA_STORE_DIR, "aliases")
 
 MODULE_TYPE_KEY = "module_type"
 """The key to specify the type of a module."""
@@ -147,16 +145,17 @@ COLOR_LIST = [
 VOID_KIARA_ID = uuid.UUID("00000000-0000-0000-0000-000000000000")
 NOT_SET_VALUE_ID = uuid.UUID("00000000-0000-0000-0000-000000000001")
 NONE_VALUE_ID = uuid.UUID("00000000-0000-0000-0000-000000000002")
-
+NONE_STORE_ID = uuid.UUID("00000000-0000-0000-0000-000000000003")
 ORPHAN_PEDIGREE_OUTPUT_NAME = "__orphan__"
 
 NO_MODULE_TYPE = "EXTERNAL_DATA"
 
-INVALID_HASH_MARKER = -1
+INVALID_HASH_MARKER = ""
 INVALID_SIZE_MARKER = -1
+NO_SERIALIZATION_MARKER = "-- serialization not supported --"
 KIARA_ROOT_TYPE_NAME = "__kiara__"
 
-SERIALIZED_DATA_TYPE_NAME = "serialized_value"
+SERIALIZED_DATA_TYPE_NAME = "serialized_data"
 LOAD_CONFIG_DATA_TYPE_NAME = "load_config"
 
 PYDANTIC_USE_CONSTRUCT: bool = False
@@ -166,6 +165,7 @@ KIARA_HASH_FUNCTION = mmh3.hash
 
 ANY_TYPE_NAME = "any"
 
+LOAD_CONFIG_PLACEHOLDER = "__placeholder__"
 
 DATA_TYPE_CATEGORY_ID = "metadata.type"
 DATA_TYPES_CATEGORY_ID = "data_types"
@@ -184,6 +184,8 @@ MODULE_CONFIG_METADATA_CATEGORY_ID = "metadata.module_config"
 MODULE_TYPE_CATEGORY_ID = "metadata.module"
 MODULE_TYPES_CATEGORY_ID = "modules"
 
+BATCH_CONFIG_TYPE_CATEGORY_ID = "instance.batch_config"
+
 PIPELINE_TYPE_CATEGORY_ID = "metadata.pipeline"
 PIPELINE_TYPES_CATEGORY_ID = "pipelines"
 PIPELINE_STEP_TYPE_CATEGORY_ID = "instance.pipeline_step"
@@ -225,3 +227,4 @@ FILE_BUNDLE_MODEL_CATEOGORY_ID = "instance.model.file_bundle"
 
 ARRAY_MODEL_CATEOGORY_ID = "instance.model.array"
 TABLE_MODEL_CATEOGORY_ID = "instance.model.table"
+DEFAULT_CONTEXT_NAME = "default"
