@@ -8,13 +8,22 @@
 """Data-related sub-commands for the cli."""
 import rich_click as click
 import shutil
+import structlog
 import sys
+from typing import Optional
 
 from kiara import Kiara
+from kiara.interfaces.tui.pager import PagerApp
+from kiara.models.module.operation import Operation
 from kiara.models.values.info import RENDER_FIELDS, ValueInfo, ValuesInfo
+from kiara.operations.included_core_operations.render_value import (
+    RenderValueOperationType,
+)
 from kiara.utils import StringYAML, is_debug, is_develop, log_message
 from kiara.utils.cli import output_format_option, terminal_print, terminal_print_model
 
+logger = structlog.getLogger()
+
 yaml = StringYAML()
 
 
@@ -33,51 +42,67 @@ def data(ctx):
     default=False,
 )
 @click.option(
-    "--internal",
+    "--include-internal",
     "-I",
-    help="Also list values that are used mostly internally (e.g. metadata for other values, ...).",
+    help="Also list values that are used mostly internally (e.g. metadata for other values, ...). Implies 'all-ids' is 'True'.",
     is_flag=True,
 )
 @click.option(
-    "--show-value_id",
+    "--value_id",
     "-i",
     help="Display value id information for each value.",
     default=False,
     is_flag=True,
 )
 @click.option(
-    "--show-pedigree",
-    "-p",
+    "--type-config",
+    "-c",
+    help="Display type details for each value.",
+    default=False,
+    is_flag=True,
+)
+@click.option(
+    "--pedigree",
+    "-P",
     help="Display pedigree information for each value.",
     default=False,
     is_flag=True,
 )
 @click.option(
-    "--show-data",
+    "--data",
     "-d",
     help="Show a preview of the data associated with this value.",
     default=False,
     is_flag=True,
 )
 @click.option(
-    "--show-load-config", "-l", help="Display this values' load config.", is_flag=True
+    "--serialized",
+    "-s",
+    help="Display serialization details for this value.",
+    is_flag=True,
 )
+@click.option("--properties", "-p", help="Display the value properties.", is_flag=True)
 @output_format_option()
 @click.pass_context
 def list_values(
     ctx,
     format,
     all_ids,
-    internal,
-    show_value_id,
-    show_pedigree,
-    show_data,
-    show_load_config,
+    include_internal,
+    value_id,
+    pedigree,
+    data,
+    type_config,
+    serialized,
+    properties,
 ):
     """List all data items that are stored in kiara."""
 
     kiara_obj: Kiara = ctx.obj["kiara"]
 
+    if include_internal:
+        all_ids = True
+
     if not all_ids:
         alias_registry = kiara_obj.alias_registry
         value_ids = [v.value_id for v in alias_registry.aliases.values()]
@@ -93,21 +118,25 @@ def list_values(
         render_fields[0] = "aliases"
         render_fields[1] = "value_id"
 
-    if not show_value_id and not all_ids:
+    if not value_id and not all_ids:
         render_fields.remove("value_id")
-    if show_data:
+    if type_config:
+        render_fields.append("data_type_config")
+    if data:
         render_fields.append("data")
-    if show_pedigree:
+    if properties:
+        render_fields.append("properties")
+    if pedigree:
         render_fields.append("pedigree")
-    if show_load_config:
-        render_fields.append("load_config")
+    if serialized:
+        render_fields.append("serialize_details")
 
     values_info_model = ValuesInfo.create_from_values(kiara_obj, *value_ids)
 
     render_config = {
         "render_type": "terminal",
         "list_by_alias": list_by_alias,
-        "show_internal": internal,
+        "show_internal": include_internal,
         "render_fields": render_fields,
     }
 
@@ -134,7 +163,13 @@ def list_values(
     "--pedigree", "-P", help="Display pedigree information for the value.", is_flag=True
 )
 @click.option(
-    "--load-config", "-l", help="Display this values' load config.", is_flag=True
+    "--lineage", "-l", help="Display lineage information for the value.", is_flag=True
+)
+@click.option(
+    "--serialized",
+    "-s",
+    help="Display this values' serialization information.",
+    is_flag=True,
 )
 @click.option("--preview-data", "-d", help="Display a data preview.", is_flag=True)
 @click.option(
@@ -162,25 +197,30 @@ def explain_value(
     value_id: str,
     metadata: bool,
     pedigree: bool,
-    load_config: bool,
+    serialized: bool,
     format: str,
     preview_data: bool,
     properties: bool,
     destinies: bool,
     destiny_backlinks: bool,
+    lineage: bool,
 ):
-    """Print the metadata of a stored value."""
+    """Print the metadata of a stored value.
+
+    All of the 'show-additional-information' flags are only applied when the 'terminal' output format is selected. This might change in the future.
+    """
 
     kiara_obj: Kiara = ctx.obj["kiara"]
 
     render_config = {
         "show_metadata": metadata,
         "show_pedigree": pedigree,
-        "show_load_config": load_config,
+        "show_serialized": serialized,
         "show_data_preview": preview_data,
         "show_properties": properties,
         "show_destinies": destinies,
         "show_destiny_backlinks": destiny_backlinks,
+        "show_lineage": lineage,
     }
 
     all_values = []
@@ -237,32 +277,52 @@ def explain_value(
 
 @data.command(name="load")
 @click.argument("value_id", nargs=1, required=True)
+@click.option(
+    "--single-page",
+    "-s",
+    help="Only pretty print a single (preview) page, instead of using a pager when available.",
+    is_flag=True,
+)
 @click.pass_context
-def load_value(ctx, value_id: str):
+def load_value(ctx, value_id: str, single_page: bool):
     """Load a stored value and print it in a format suitable for the terminal."""
 
     kiara_obj: Kiara = ctx.obj["kiara"]
 
     value = kiara_obj.data_registry.get_value(value_id=value_id)
-    # if value is None:
-    #     print(f"No value available for id: {value_id}")
-    #     sys.exit(1)
 
-    try:
-        renderable = kiara_obj.data_registry.render_data(
-            value.value_id, target_type="terminal_renderable"
+    render_op: Optional[Operation] = None
+    if not single_page:
+        render_value_op_type: RenderValueOperationType = kiara_obj.operation_registry.get_operation_type("render_value")  # type: ignore
+        render_op = render_value_op_type.get_render_operation(
+            source_type=value.data_type_name, target_type="terminal_renderable"
         )
-    except Exception as e:
 
-        if is_debug():
-            import traceback
+    if not render_op:
+        logger.debug(
+            "fallback.render_value",
+            solution="use pretty print",
+            source_type=value.data_type_name,
+            target_type="terminal_renderable",
+            reason="no 'render_value' operation for source/target operation",
+        )
+        try:
+            renderable = kiara_obj.data_registry.pretty_print_data(
+                value.value_id, target_type="terminal_renderable"
+            )
+        except Exception as e:
 
-            traceback.print_exc()
-        log_message("error.render_value", value=value.value_id, error=e)
+            if is_debug():
+                import traceback
 
-        renderable = [str(value.data)]
+                traceback.print_exc()
+            log_message("error.pretty_print", value=value.value_id, error=e)
+            renderable = [str(value.data)]
 
-    terminal_print(renderable)
+        terminal_print(renderable)
+        sys.exit(0)
+
+    PagerApp.run(kiara=kiara_obj, value=value, operation=render_op)
 
 
 if is_develop():
