@@ -13,6 +13,7 @@ import sys
 from pathlib import Path
 from pydantic import ValidationError
 from rich.console import Group
+from rich.rule import Rule
 from typing import Dict, Iterable, List
 
 from kiara import Kiara
@@ -21,7 +22,7 @@ from kiara.exceptions import (
     InvalidValuesException,
     NoSuchExecutionTargetException,
 )
-from kiara.interfaces.python_api import KiaraOperation
+from kiara.interfaces.python_api.operation import KiaraOperation
 from kiara.utils import dict_from_cli_args, is_debug
 from kiara.utils.cli import terminal_print
 from kiara.utils.output import OutputDetails, create_table_from_base_model_cls
@@ -63,6 +64,7 @@ def run(
     explain: bool,
     save: Iterable[str],
 ):
+    """Run a kiara operation."""
 
     # =========================================================================
     # initialize a few variables
@@ -155,12 +157,15 @@ def run(
         terminal_print(msg, in_panel="[b red]Module configuration error[/b red]")
         sys.exit(1)
     except Exception as e:
-        terminal_print()
         if is_debug():
             import traceback
 
             traceback.print_exc()
-        terminal_print(str(e))
+        terminal_print()
+        terminal_print(
+            f"Error when trying to validate the operation [i]'{kiara_op.operation_name}'[/i]:\n"
+        )
+        terminal_print(f"    [red]{e}[/red]")
         sys.exit(1)
 
     # =========================================================================
@@ -202,21 +207,36 @@ def run(
     try:
         operation_inputs = kiara_op.operation_inputs
     except InvalidValuesException as ive:
-        print()
-        terminal_print(str(ive))
-        print()
-        print("Details:\n")
-        for k, v in ive.invalid_inputs.items():
-            terminal_print(f"  - [b]{k}[/b]: [i]{v}[/i]")
+
+        terminal_print()
+        rg = Group(
+            "",
+            f"Can't run operation: {ive}",
+            "",
+            Rule(),
+            "",
+            kiara_op.create_renderable(
+                show_operation_name=True, show_inputs=True, show_outputs_schema=True
+            ),
+        )
+        terminal_print(rg, in_panel=f"Run info: [b]{kiara_op.operation_name}[/b]")
         sys.exit(1)
 
     invalid = operation_inputs.check_invalid()
     if invalid:
-        print()
-        print("Can't create operation inputs, invalid field(s):")
-        print()
-        for k, v in invalid.items():
-            terminal_print(f"  - [b]{k}[/b]: [i]{v}[/i]")
+
+        terminal_print()
+        rg = Group(
+            "",
+            "Can't run operation, invalid or insufficient inputs.",
+            "",
+            Rule(),
+            "",
+            kiara_op.create_renderable(
+                show_operation_name=True, show_inputs=True, show_outputs_schema=True
+            ),
+        )
+        terminal_print(rg, in_panel=f"Run info: [b]{kiara_op.operation_name}[/b]")
         sys.exit(1)
 
     # =========================================================================
@@ -231,16 +251,19 @@ def run(
         sys.exit(1)
 
     if not silent:
-        print()
         if len(outputs) > 1:
-            terminal_print("[b]Results:[/b]")
+            title = "[b]Results[/b]"
         else:
-            terminal_print("[b]Result:[/b]")
+            title = "[b]Result[/b]"
 
-        for field_name, value in outputs.items():
-            print()
-            terminal_print(f"* [b i]{field_name}[/b i]")
-            terminal_print(kiara_obj.data_registry.render_data(value.value_id))
+        # for field_name, value in outputs.items():
+        #     results.append("")
+        #     results.append(f"* [b i]{field_name}[/b i]")
+        #     results.append(kiara_obj.data_registry.render_data(value.value_id))
+
+        terminal_print(
+            outputs, in_panel=title, empty_line_before=True, show_data_type=True
+        )
 
     # for k, v in outputs.items():
     #     rendered = kiara_obj.data_registry.render_data(v)
@@ -250,6 +273,8 @@ def run(
 
         save_results = kiara_op.save_result(job_id=job_id, aliases=final_aliases)
         if not silent:
-            print()
-            terminal_print("[b]Stored value(s):[/b]")
-            terminal_print(save_results)
+            if len(save_results) == 1:
+                title = "[b]Stored result value[/b]"
+            else:
+                title = "[b]Stored result values[/b]"
+            terminal_print(save_results, in_panel=title, empty_line_before=True)
