@@ -20,33 +20,22 @@ from typing import (
     Any,
     Dict,
     Iterable,
+    List,
     Literal,
     Mapping,
     Optional,
-    Set,
     Type,
     Union,
 )
 
-from kiara.defaults import (
-    OPERATION_CATEOGORY_ID,
-    OPERATION_CONFIG_CATEOGORY_ID,
-    OPERATION_DETAILS_CATEOGORY_ID,
-    OPERATION_TYPE_CATEGORY_ID,
-    PYDANTIC_USE_CONSTRUCT,
-)
+from kiara.defaults import PYDANTIC_USE_CONSTRUCT
 from kiara.models import KiaraModel
 from kiara.models.documentation import (
     AuthorsMetadataModel,
     ContextMetadataModel,
     DocumentationMetadataModel,
 )
-from kiara.models.info import (
-    InfoModelGroup,
-    KiaraInfoModel,
-    KiaraTypeInfoModel,
-    TypeInfoModelGroup,
-)
+from kiara.models.info import InfoModelGroup, ItemInfo, TypeInfo, TypeInfoModelGroup
 from kiara.models.module import KiaraModuleClass, KiaraModuleTypeInfo
 from kiara.models.module.jobs import JobConfig
 from kiara.models.module.manifest import Manifest
@@ -94,6 +83,9 @@ class OperationSchema(InputOutputObject):
 
 
 class OperationDetails(KiaraModel):
+
+    _kiara_model_id = "instance.operation_details"
+
     @classmethod
     def create_operation_details(cls, **details: Any):
 
@@ -113,12 +105,6 @@ class OperationDetails(KiaraModel):
     def _retrieve_id(self) -> str:
         return self.operation_id
 
-    def _retrieve_category_id(self) -> str:
-        return OPERATION_DETAILS_CATEOGORY_ID
-
-    def _retrieve_data_to_hash(self) -> Any:
-        return self.dict()
-
     @property
     def inputs_schema(self) -> Mapping[str, ValueSchema]:
         """The input schema for this module."""
@@ -143,6 +129,8 @@ class OperationDetails(KiaraModel):
 
 class BaseOperationDetails(OperationDetails):
 
+    _kiara_model_id = "instance.operation_details.base"
+
     _op_schema: OperationSchema = PrivateAttr(default=None)
 
     def retrieve_inputs_schema(cls) -> ValueSetSchema:
@@ -174,15 +162,6 @@ class OperationConfig(KiaraModel):
     def validate_doc(cls, value):
         return DocumentationMetadataModel.create(value)
 
-    def _retrieve_id(self) -> str:
-        return str(self.model_data_hash)
-
-    def _retrieve_category_id(self) -> str:
-        return OPERATION_CONFIG_CATEOGORY_ID
-
-    def _retrieve_data_to_hash(self) -> Any:
-        return self.dict()
-
     @abc.abstractmethod
     def retrieve_module_type(self, kiara: "Kiara") -> str:
         pass
@@ -194,6 +173,8 @@ class OperationConfig(KiaraModel):
 
 class ManifestOperationConfig(OperationConfig):
 
+    _kiara_model_id = "instance.operation_config.manifest"
+
     module_type: str = Field(description="The module type.")
     module_config: Dict[str, Any] = Field(
         default_factory=dict, description="The configuration for the module."
@@ -208,12 +189,17 @@ class ManifestOperationConfig(OperationConfig):
 
 class PipelineOperationConfig(OperationConfig):
 
-    pipeline_id: str = Field(description="The pipeline id.")
+    _kiara_model_id = "instance.operation_config.pipeline"
+
+    pipeline_name: str = Field(description="The pipeline id.")
     pipeline_config: Mapping[str, Any] = Field(description="The pipeline config data.")
     module_map: Dict[str, Any] = Field(
         description="A lookup map to resolves module names to operations.",
         default_factory=dict,
     )
+    metadata: Mapping[str, Any] = Field(
+        description="Additional metadata for the pipeline.", default_factory=dict
+    )
 
     @validator("pipeline_config")
     def validate_pipeline_config(cls, value):
@@ -228,8 +214,9 @@ class PipelineOperationConfig(OperationConfig):
 
     def retrieve_module_config(self, kiara: "Kiara") -> Mapping[str, Any]:
 
-        pipeline_config = PipelineConfig.from_config(
-            pipeline_id=self.pipeline_id,
+        # using _from_config here because otherwise we'd enter an infinite loop
+        pipeline_config = PipelineConfig._from_config(
+            pipeline_name=self.pipeline_name,
             data=self.pipeline_config,
             kiara=kiara,
             module_map=self.module_map,
@@ -243,27 +230,36 @@ class PipelineOperationConfig(OperationConfig):
 
     def __repr__(self):
 
-        return f"{self.__class__.__name__}(pipeline_id={self.pipeline_id} required_modules={list(self.required_module_types)} model_id={self.model_id}, category={self.category_id}, fields=[{', '.join(self.__fields__.keys())}])"
+        return f"{self.__class__.__name__}(pipeline_name={self.pipeline_name} required_modules={list(self.required_module_types)} instance_id={self.instance_id}, category={self.model_type_id}, fields=[{', '.join(self.__fields__.keys())}])"
 
 
 class Operation(Manifest):
+
+    _kiara_model_id = "instance.operation"
+
     @classmethod
-    def create_from_module(cls, module: KiaraModule) -> "Operation":
+    def create_from_module(cls, module: KiaraModule, doc: Any = None) -> "Operation":
 
         from kiara.operations.included_core_operations import (
             CustomModuleOperationDetails,
         )
 
-        op_id = f"{module.module_type_name}._{module.module_instance_hash}"
+        op_id = f"{module.module_type_name}._{module.module_instance_cid}"
 
         details = CustomModuleOperationDetails.create_from_module(module=module)
+
+        if doc is not None:
+            doc = DocumentationMetadataModel.create(doc)
+        else:
+            doc = DocumentationMetadataModel.from_class_doc(module.__class__)
+
         operation = Operation(
             module_type=module.module_type_name,
             module_config=module.config.dict(),
             operation_id=op_id,
             operation_details=details,
             module_details=KiaraModuleClass.from_module(module),
-            doc=DocumentationMetadataModel.from_class_doc(module.__class__),
+            doc=doc,
         )
         operation._module = module
         return operation
@@ -279,17 +275,14 @@ class Operation(Manifest):
     module_details: KiaraModuleClass = Field(
         description="The class of the underlying module."
     )
+    metadata: Mapping[str, Any] = Field(
+        description="Additional metadata for this operation.", default_factory=dict
+    )
 
     _module: Optional["KiaraModule"] = PrivateAttr(default=None)
 
-    def _retrieve_id(self) -> str:
-        return str(self.model_data_hash)
-
-    def _retrieve_category_id(self) -> str:
-        return OPERATION_CATEOGORY_ID
-
     def _retrieve_data_to_hash(self) -> Any:
-        raise NotImplementedError()
+        return {"operation_id": self.operation_id, "manifest": self.manifest_cid}
 
     @property
     def module(self) -> "KiaraModule":
@@ -358,6 +351,9 @@ class Operation(Manifest):
 
         include_full_doc = config.get("include_full_doc", True)
         include_src = config.get("include_src", False)
+        include_inputs = config.get("include_inputs", True)
+        include_outputs = config.get("include_outputs", True)
+        include_module_details = config.get("include_module_details", False)
 
         table = Table(box=box.SIMPLE, show_header=False, show_lines=True)
         table.add_column("Property", style="i")
@@ -371,58 +367,46 @@ class Operation(Manifest):
 
         # module_type_md = self.module.get_type_metadata()
 
-        inputs_table = create_table_from_field_schemas(
-            _add_required=True,
-            _add_default=True,
-            _show_header=True,
-            _constants=None,
-            **self.operation_details.inputs_schema,
-        )
-        # constants = self.module_config.get("constants")
-        # inputs_table = create_table_from_field_schemas(
-        #     _add_required=True,
-        #     _add_default=True,
-        #     _show_header=True,
-        #     _constants=constants,
-        #     **self.module.inputs_schema,
-        # )
-        table.add_row("Inputs", inputs_table)
-        outputs_table = create_table_from_field_schemas(
-            _add_required=False,
-            _add_default=False,
-            _show_header=True,
-            _constants=None,
-            **self.operation_details.outputs_schema,
-        )
-        # outputs_table = create_table_from_field_schemas(
-        #     _add_required=False,
-        #     _add_default=False,
-        #     _show_header=True,
-        #     _constants=None,
-        #     **self.module.outputs_schema,
-        # )
-        table.add_row("Outputs", outputs_table)
+        if include_inputs:
+            inputs_table = create_table_from_field_schemas(
+                _add_required=True,
+                _add_default=True,
+                _show_header=True,
+                _constants=None,
+                **self.operation_details.inputs_schema,
+            )
+            table.add_row("Inputs", inputs_table)
+        if include_outputs:
+            outputs_table = create_table_from_field_schemas(
+                _add_required=False,
+                _add_default=False,
+                _show_header=True,
+                _constants=None,
+                **self.operation_details.outputs_schema,
+            )
+            table.add_row("Outputs", outputs_table)
 
-        table.add_row("Module type", self.module_type)
+        if include_module_details:
+            table.add_row("Module type", self.module_type)
 
-        module_config = self.module.config.json(option=orjson.OPT_INDENT_2)
-        conf = Syntax(
-            module_config,
-            "json",
-            background_color="default",
-        )
-        table.add_row("Module config", conf)
+            module_config = self.module.config.json(option=orjson.OPT_INDENT_2)
+            conf = Syntax(
+                module_config,
+                "json",
+                background_color="default",
+            )
+            table.add_row("Module config", conf)
 
-        module_type_md = KiaraModuleTypeInfo.create_from_type_class(
-            self.module_details.get_class()  # type: ignore
-        )
+            module_type_md = KiaraModuleTypeInfo.create_from_type_class(
+                self.module_details.get_class()  # type: ignore
+            )
 
-        desc = module_type_md.documentation.description
-        module_md = module_type_md.create_renderable(
-            include_doc=False, include_src=False, include_config_schema=False
-        )
-        m_md = Group(desc, module_md)
-        table.add_row("Module metadata", m_md)
+            desc = module_type_md.documentation.description
+            module_md = module_type_md.create_renderable(
+                include_doc=False, include_src=False, include_config_schema=False
+            )
+            m_md = Group(desc, module_md)
+            table.add_row("Module metadata", m_md)
 
         if include_src:
             table.add_row("Source code", module_type_md.process_src)
@@ -430,7 +414,10 @@ class Operation(Manifest):
         return table
 
 
-class OperationTypeInfo(KiaraTypeInfoModel):
+class OperationTypeInfo(TypeInfo):
+
+    _kiara_model_id = "info.operation_type"
+
     @classmethod
     def create_from_type_class(
         cls, type_cls: Type["OperationType"]
@@ -464,16 +451,16 @@ class OperationTypeInfo(KiaraTypeInfoModel):
     def _retrieve_id(self) -> str:
         return self.type_name
 
-    def _retrieve_category_id(self) -> str:
-        return OPERATION_TYPE_CATEGORY_ID
-
     def _retrieve_data_to_hash(self) -> Any:
         return self.type_name
 
 
 class OperationTypeClassesInfo(TypeInfoModelGroup):
+
+    _kiara_model_id = "info.operation_types"
+
     @classmethod
-    def base_info_class(cls) -> Type[KiaraTypeInfoModel]:
+    def base_info_class(cls) -> Type[TypeInfo]:
         return OperationTypeInfo
 
     type_name: Literal["operation_type"] = "operation_type"
@@ -482,7 +469,10 @@ class OperationTypeClassesInfo(TypeInfoModelGroup):
     )
 
 
-class OperationInfo(KiaraInfoModel):
+class OperationInfo(ItemInfo):
+
+    _kiara_model_id = "info.operation"
+
     @classmethod
     def create_from_operation(cls, kiara: "Kiara", operation: Operation):
 
@@ -498,7 +488,7 @@ class OperationInfo(KiaraInfoModel):
 
         op_info = OperationInfo.construct(
             type_name=operation.operation_id,
-            operation_types=op_types,
+            operation_types=list(op_types),
             operation=operation,
             documentation=operation.doc,
             authors=authors_md,
@@ -512,7 +502,7 @@ class OperationInfo(KiaraInfoModel):
         return "operation"
 
     operation: Operation = Field(description="The operation instance.")
-    operation_types: Set[str] = Field(
+    operation_types: List[str] = Field(
         description="The operation types this operation belongs to."
     )
 
@@ -537,8 +527,11 @@ class OperationInfo(KiaraInfoModel):
 
 
 class OperationGroupInfo(InfoModelGroup):
+
+    _kiara_model_id = "info.operations"
+
     @classmethod
-    def base_info_class(cls) -> Type[KiaraInfoModel]:
+    def base_info_class(cls) -> Type[ItemInfo]:
         return OperationInfo
 
     @classmethod
@@ -574,9 +567,9 @@ class OperationGroupInfo(InfoModelGroup):
         filter = config.get("filter", [])
 
         table = Table(box=box.SIMPLE, show_header=True)
-        table.add_column("Id", no_wrap=True, style="b")
+        table.add_column("Id", no_wrap=True, style="i")
         table.add_column("Type(s)", style="green")
-        table.add_column("Description", style="i")
+        table.add_column("Description")
 
         for op_id, op_info in self.type_infos.items():
 
@@ -588,10 +581,8 @@ class OperationGroupInfo(InfoModelGroup):
 
             types = op_info.operation_types
 
-            try:
+            if "custom_module" in types:
                 types.remove("custom_module")
-            except KeyError:
-                pass
 
             desc_str = op_info.documentation.description
             if full_doc:
@@ -640,7 +631,7 @@ class OperationGroupInfo(InfoModelGroup):
 
         table = Table(box=box.SIMPLE, show_header=True)
         table.add_column("Type", no_wrap=True, style="b green")
-        table.add_column("Id", no_wrap=True)
+        table.add_column("Id", no_wrap=True, style="i")
         if full_doc:
             table.add_column("Documentation", no_wrap=False, style="i")
         else:
