@@ -7,11 +7,9 @@
 
 import networkx as nx
 from functools import lru_cache
-from networkx import NetworkXNoPath, NodeNotFound
 from pydantic import Field, PrivateAttr, root_validator
-from typing import Any, Dict, Iterable, List, Mapping, Optional, Set
+from typing import Any, Dict, Iterable, List, Mapping, Set
 
-from kiara.defaults import PIPELINE_STRUCTURE_TYPE_CATEGORY_ID
 from kiara.models import KiaraModel
 from kiara.models.module.pipeline import PipelineConfig, PipelineStep
 from kiara.models.module.pipeline.value_refs import (
@@ -30,82 +28,84 @@ def generate_pipeline_endpoint_name(step_id: str, value_name: str):
     return f"{step_id}__{value_name}"
 
 
-def calculate_shortest_field_aliases(
-    steps: List[PipelineStep], alias_type: str, alias_for: str
-):
-    """Utility method to figure out the best field aliases automatically."""
-
-    assert alias_for in ["inputs", "outputs"]
-    if alias_type == "auto_all_outputs":
-
-        aliases: Dict[str, List[str]] = {}
-
-        for step in steps:
-
-            if alias_for == "inputs":
-                field_names = step.module.input_names
-            else:
-                field_names = step.module.output_names
-
-            for field_name in field_names:
-                aliases.setdefault(field_name, []).append(step.step_id)
-
-        result = {}
-        for field_name, step_ids in aliases.items():
-            if len(step_ids) == 1:
-                result[
-                    generate_pipeline_endpoint_name(step_ids[0], field_name)
-                ] = field_name
-            else:
-                for step_id in step_ids:
-                    generated = generate_pipeline_endpoint_name(step_id, field_name)
-                    result[generated] = generated
-
-    elif alias_type == "auto":
-
-        aliases = {}
-
-        for stage_nr, step in enumerate(steps):
-
-            _field_names: Optional[Iterable[str]] = None
-            if alias_for == "inputs":
-                _field_names = step.module.input_names
-            else:
-                if stage_nr == len(steps) - 1:
-                    _field_names = step.module.output_names
-
-            if not _field_names:
-                continue
-
-            for field_name in _field_names:
-                aliases.setdefault(field_name, []).append(step.step_id)
-
-        result = {}
-        for field_name, step_ids in aliases.items():
-            if len(step_ids) == 1:
-                result[
-                    generate_pipeline_endpoint_name(step_ids[0], field_name)
-                ] = field_name
-            else:
-                for step_id in step_ids:
-                    generated = generate_pipeline_endpoint_name(step_id, field_name)
-                    result[generated] = generated
-
-    return result
+# def calculate_shortest_field_aliases(
+#     steps: List[PipelineStep], alias_type: str, alias_for: str
+# ):
+#     """Utility method to figure out the best field aliases automatically."""
+#
+#     assert alias_for in ["inputs", "outputs"]
+#     if alias_type == "auto_all_outputs":
+#
+#         aliases: Dict[str, List[str]] = {}
+#
+#         for step in steps:
+#
+#             if alias_for == "inputs":
+#                 field_names = step.module.input_names
+#             else:
+#                 field_names = step.module.output_names
+#
+#             for field_name in field_names:
+#                 aliases.setdefault(field_name, []).append(step.step_id)
+#
+#         result = {}
+#         for field_name, step_ids in aliases.items():
+#             if len(step_ids) == 1:
+#                 result[
+#                     generate_pipeline_endpoint_name(step_ids[0], field_name)
+#                 ] = field_name
+#             else:
+#                 for step_id in step_ids:
+#                     generated = generate_pipeline_endpoint_name(step_id, field_name)
+#                     result[generated] = generated
+#
+#     elif alias_type == "auto":
+#
+#         aliases = {}
+#
+#         for stage_nr, step in enumerate(steps):
+#
+#             _field_names: Optional[Iterable[str]] = None
+#             if alias_for == "inputs":
+#                 _field_names = step.module.input_names
+#             else:
+#                 if stage_nr == len(steps) - 1:
+#                     _field_names = step.module.output_names
+#
+#             if not _field_names:
+#                 continue
+#
+#             for field_name in _field_names:
+#                 aliases.setdefault(field_name, []).append(step.step_id)
+#
+#         result = {}
+#         for field_name, step_ids in aliases.items():
+#             if len(step_ids) == 1:
+#                 result[
+#                     generate_pipeline_endpoint_name(step_ids[0], field_name)
+#                 ] = field_name
+#             else:
+#                 for step_id in step_ids:
+#                     generated = generate_pipeline_endpoint_name(step_id, field_name)
+#                     result[generated] = generated
+#
+#     return result
 
 
-ALLOWED_INPUT_ALIAS_MARKERS = ["auto", "auto_all_outputs"]
+# ALLOWED_INPUT_ALIAS_MARKERS = ["auto", "auto_all_outputs"]
 
 
 class PipelineStructure(KiaraModel):
     """An object that holds one or several steps, and describes the connections between them."""
 
+    _kiara_model_id = "instance.pipeline_structure"
+
     pipeline_config: PipelineConfig = Field(
         description="The underlying pipeline config."
     )
     steps: List[PipelineStep] = Field(description="The pipeline steps ")
-    input_aliases: Dict[str, str] = Field(description="The (resolved) input aliases.")
-    output_aliases: Dict[str, str] = Field(description="The (resolved) output aliases.")
+    input_aliases: Dict[str, str] = Field(description="The input aliases.")
+    output_aliases: Dict[str, str] = Field(description="The output aliases.")
 
     @root_validator(pre=True)
     def validate_pipeline_config(cls, values):
@@ -122,39 +122,42 @@ class PipelineStructure(KiaraModel):
         _config: PipelineConfig = pipeline_config
         _steps: List[PipelineStep] = list(_config.steps)
 
-        if not _config.input_aliases:
-            input_aliases = {}
-        else:
-            if isinstance(_config.input_aliases, str):
-                if _config.input_aliases not in ALLOWED_INPUT_ALIAS_MARKERS:
-                    raise Exception(
-                        f"Can't create pipeline, invalid value '{_config.input_aliases}' for 'input_aliases'. Either specify a dict, or use one of: {', '.join(ALLOWED_INPUT_ALIAS_MARKERS)}"
-                    )
+        _input_aliases: Dict[str, str] = dict(_config.input_aliases)
+        _output_aliases: Dict[str, str] = dict(_config.output_aliases)
 
-                input_aliases = calculate_shortest_field_aliases(
-                    _steps, _config.input_aliases, "inputs"
-                )
-            else:
-                input_aliases = dict(_config.input_aliases)
+        invalid_input_aliases = [a for a in _input_aliases.values() if "." in a]
+        if invalid_input_aliases:
+            raise Exception(
+                f"Invalid input aliases, aliases can't contain special characters: {', '.join(invalid_input_aliases)}."
+            )
+        invalid_output_aliases = [a for a in _input_aliases.values() if "." in a]
+        if invalid_input_aliases:
+            raise Exception(
+                f"Invalid input aliases, aliases can't contain special characters: {', '.join(invalid_output_aliases)}."
+            )
 
-        _input_aliases: Dict[str, str] = dict(input_aliases)  # type: ignore
-
-        if not _config.output_aliases:
-            output_aliases = {}
-        else:
-            if isinstance(_config.output_aliases, str):
-                if _config.output_aliases not in ALLOWED_INPUT_ALIAS_MARKERS:
-                    raise Exception(
-                        f"Can't create pipeline, invalid value '{_config.output_aliases}' for 'output_aliases'. Either specify a dict, or use one of: {', '.join(ALLOWED_INPUT_ALIAS_MARKERS)}"
-                    )
-
-                output_aliases = calculate_shortest_field_aliases(
-                    _steps, _config.output_aliases, "outputs"
-                )
-            else:
-                output_aliases = dict(_config.output_aliases)
-
-        _output_aliases: Dict[str, str] = output_aliases
+        valid_input_names = set()
+        for step in _steps:
+            for input_name in step.module.input_names:
+                valid_input_names.add(f"{step.step_id}.{input_name}")
+        invalid_input_aliases = [
+            a for a in _input_aliases.keys() if a not in valid_input_names
+        ]
+        if invalid_input_aliases:
+            raise Exception(
+                f"Invalid input reference(s): {', '.join(invalid_input_aliases)}. Must be one of: {', '.join(valid_input_names)}"
+            )
+        valid_output_names = set()
+        for step in _steps:
+            for output_name in step.module.output_names:
+                valid_output_names.add(f"{step.step_id}.{output_name}")
+        invalid_output_names = [
+            a for a in _output_aliases.keys() if a not in valid_output_names
+        ]
+        if invalid_output_names:
+            raise Exception(
+                f"Invalid output reference(s): {', '.join(invalid_output_names)}. Must be one of: {', '.join(valid_output_names)}"
+            )
 
         values["steps"] = _steps
         values["input_aliases"] = _input_aliases
@@ -177,16 +180,13 @@ class PipelineStructure(KiaraModel):
 
     def _retrieve_data_to_hash(self) -> Any:
         return {
-            "steps": [step.model_data_hash for step in self.steps],
+            "steps": [step.instance_cid for step in self.steps],
             "input_aliases": self.input_aliases,
             "output_aliases": self.output_aliases,
         }
 
     def _retrieve_id(self) -> str:
-        return self.pipeline_config.model_id
-
-    def _retrieve_category_id(self) -> str:
-        return PIPELINE_STRUCTURE_TYPE_CATEGORY_ID
+        return self.pipeline_config.instance_id
 
     @property
     def steps_details(self) -> Mapping[str, Any]:
@@ -313,10 +313,11 @@ class PipelineStructure(KiaraModel):
     @property
     def pipeline_inputs_schema(self) -> Mapping[str, ValueSchema]:
 
-        return {
+        schemas = {
             input_name: w_in.value_schema
             for input_name, w_in in self.pipeline_input_refs.items()
         }
+        return schemas
 
     @property
     def pipeline_outputs_schema(self) -> Mapping[str, ValueSchema]:
@@ -391,16 +392,18 @@ class PipelineStructure(KiaraModel):
                 step_alias = generate_step_alias(step.step_id, output_name)
                 outputs[step_alias] = step_output
 
-                step_output_name = generate_pipeline_endpoint_name(
-                    step_id=step.step_id, value_name=output_name
-                )
-                if self.output_aliases:
-                    if step_output_name in self.output_aliases.keys():
-                        step_output_name = self.output_aliases[step_output_name]
-                    else:
-                        if not self._add_all_workflow_outputs:
-                            # this output is not interesting for the workflow
-                            step_output_name = None
+                # step_output_name = generate_pipeline_endpoint_name(
+                #     step_id=step.step_id, value_name=output_name
+                # )
+                step_output_name = f"{step.step_id}.{output_name}"
+                if not self.output_aliases:
+                    raise NotImplementedError()
+                if step_output_name in self.output_aliases.keys():
+                    step_output_name = self.output_aliases[step_output_name]
+                else:
+                    if not self._add_all_workflow_outputs:
+                        # this output is not interesting for the workflow
+                        step_output_name = None
 
                 if step_output_name:
                     step_output_address = StepValueAddress(
@@ -492,16 +495,17 @@ class PipelineStructure(KiaraModel):
 
                 else:
                     # this means we connect to pipeline input
-                    pipeline_input_name = generate_pipeline_endpoint_name(
-                        step_id=step.step_id, value_name=input_name
-                    )
+                    # pipeline_input_name = generate_pipeline_endpoint_name(
+                    #     step_id=step.step_id, value_name=input_name
+                    # )
+                    pipeline_input_name = f"{step.step_id}.{input_name}"
                     # check whether this input has an alias associated with it
-                    if self.input_aliases:
-                        if pipeline_input_name in self.input_aliases.keys():
-                            # this means we use the pipeline alias
-                            pipeline_input_name = self.input_aliases[
-                                pipeline_input_name
-                            ]
+                    if not self.input_aliases:
+                        raise NotImplementedError()
+
+                    if pipeline_input_name in self.input_aliases.keys():
+                        # this means we use the pipeline alias
+                        pipeline_input_name = self.input_aliases[pipeline_input_name]
 
                     if pipeline_input_name in existing_pipeline_input_points.keys():
                         # we already created a pipeline input with this name
@@ -603,54 +607,54 @@ class PipelineStructure(KiaraModel):
         # don't have a valid input set, because the inputs downstream they are connecting to are 'non-required'
         # optional_steps = []
 
-        last_stage = self._processing_stages[-1]
-
-        step_nodes: List[PipelineStep] = [
-            node
-            for node in self._data_flow_graph_simple.nodes
-            if isinstance(node, PipelineStep)
-        ]
-
-        all_required_inputs = []
-        for step_id in last_stage:
-
-            step = self.get_step(step_id)
-            step_nodes.remove(step)
-
-            for k, s_inp in self.get_step_input_refs(step_id).items():
-                if not s_inp.value_schema.is_required():
-                    continue
-                all_required_inputs.append(s_inp)
-
-        for pipeline_input in self.pipeline_input_refs.values():
-
-            for last_step_input in all_required_inputs:
-                try:
-                    path = nx.shortest_path(
-                        self._data_flow_graph_simple, pipeline_input, last_step_input
-                    )
-                    for p in path:
-                        if p in step_nodes:
-                            step_nodes.remove(p)
-                except (NetworkXNoPath, NodeNotFound):
-                    pass
-                    # print("NO PATH")
-                    # print(f"{pipeline_input} -> {last_step_input}")
-
-        for s in step_nodes:
-            self._steps_details[s.step_id]["required"] = False
-            # s.required = False
-
-        for input_name, inp in self.pipeline_input_refs.items():
-            steps = set()
-            for ci in inp.connected_inputs:
-                steps.add(ci.step_id)
-
-            optional = True
-            for step_id in steps:
-                step = self.get_step(step_id)
-                if self._steps_details[step_id]["required"]:
-                    optional = False
-                    break
-            if optional:
-                inp.value_schema.optional = True
+        # last_stage = self._processing_stages[-1]
+        #
+        # step_nodes: List[PipelineStep] = [
+        #     node
+        #     for node in self._data_flow_graph_simple.nodes
+        #     if isinstance(node, PipelineStep)
+        # ]
+        #
+        # all_required_inputs = []
+        # for step_id in last_stage:
+        #
+        #     step = self.get_step(step_id)
+        #     step_nodes.remove(step)
+        #
+        #     for k, s_inp in self.get_step_input_refs(step_id).items():
+        #         if not s_inp.value_schema.is_required():
+        #             continue
+        #         all_required_inputs.append(s_inp)
+        #
+        # for pipeline_input in self.pipeline_input_refs.values():
+        #
+        #     for last_step_input in all_required_inputs:
+        #         try:
+        #             path = nx.shortest_path(
+        #                 self._data_flow_graph_simple, pipeline_input, last_step_input
+        #             )
+        #             for p in path:
+        #                 if p in step_nodes:
+        #                     step_nodes.remove(p)
+        #         except (NetworkXNoPath, NodeNotFound):
+        #             pass
+        #             # print("NO PATH")
+        #             # print(f"{pipeline_input} -> {last_step_input}")
+        #
+        # for s in step_nodes:
+        #     self._steps_details[s.step_id]["required"] = False
+        #     # s.required = False
+        #
+        # for input_name, inp in self.pipeline_input_refs.items():
+        #     steps = set()
+        #     for ci in inp.connected_inputs:
+        #         steps.add(ci.step_id)
+        #
+        #     optional = True
+        #     for step_id in steps:
+        #         step = self.get_step(step_id)
+        #         if self._steps_details[step_id]["required"]:
+        #             optional = False
+        #             break
+        #     if optional:
+        #         inp.value_schema.optional = True
