@@ -4,11 +4,10 @@
 #  Copyright (c) 2021, Markus Binsteiner
 #
 #  Mozilla Public License, version 2.0 (see LICENSE or https://www.mozilla.org/en-US/MPL/2.0/)
-
+import copy
 import structlog
 import uuid
 from rich.console import RenderableType
-from sqlalchemy.engine import Engine
 from typing import (
     TYPE_CHECKING,
     Any,
@@ -23,31 +22,33 @@ from typing import (
 )
 
 from kiara.data_types import DataType
+from kiara.data_types.included_core_types import NoneType
 from kiara.defaults import (
     INVALID_HASH_MARKER,
+    NO_SERIALIZATION_MARKER,
+    NONE_STORE_ID,
     NONE_VALUE_ID,
     NOT_SET_VALUE_ID,
     ORPHAN_PEDIGREE_OUTPUT_NAME,
     STRICT_CHECKS,
     SpecialValue,
 )
-from kiara.exceptions import InvalidValuesException, JobConfigException
+from kiara.exceptions import InvalidValuesException
 from kiara.models.events.data_registry import (
     DataArchiveAddedEvent,
     ValueCreatedEvent,
     ValuePreStoreEvent,
+    ValueRegisteredEvent,
     ValueStoredEvent,
 )
-from kiara.models.module.persistence import (
-    ByteProvisioningStrategy,
-    BytesStructure,
-    LoadConfig,
-)
+from kiara.models.module.operation import Operation
 from kiara.models.python_class import PythonClass
 from kiara.models.values import ValueStatus
 from kiara.models.values.value import (
     ORPHAN,
-    UnloadableData,
+    PersistedData,
+    SerializationMetadata,
+    SerializedData,
     Value,
     ValueMap,
     ValueMapReadOnly,
@@ -56,8 +57,8 @@ from kiara.models.values.value import (
 from kiara.models.values.value_schema import ValueSchema
 from kiara.registries.data.data_store import DataArchive, DataStore
 from kiara.registries.ids import ID_REGISTRY
-from kiara.utils import is_debug, log_message
-from kiara.utils.data import render_data
+from kiara.utils import log_exception, log_message
+from kiara.utils.data import pretty_print_data
 
 if TYPE_CHECKING:
     from kiara.context import Kiara
@@ -65,11 +66,21 @@ if TYPE_CHECKING:
 logger = structlog.getLogger()
 
 
+NONE_PERSISTED_DATA = PersistedData.construct(
+    data_type="none",
+    data_type_config={},
+    serialization_profile="none",
+    metadata=SerializationMetadata(),
+    hash_codec="sha2-256",
+    archive_id=NONE_STORE_ID,
+    chunk_id_map={},
+)
+
+
 class DataRegistry(object):
     def __init__(self, kiara: "Kiara"):
 
         self._kiara: Kiara = kiara
-        self._engine: Engine = self._kiara._engine
 
         self._event_callback: Callable = self._kiara.event_registry.add_producer(self)
 
@@ -80,13 +91,13 @@ class DataRegistry(object):
 
         self._value_archive_lookup_map: Dict[uuid.UUID, str] = {}
 
-        self._values_by_hash: Dict[int, Set[uuid.UUID]] = {}
+        self._values_by_hash: Dict[str, Set[uuid.UUID]] = {}
 
         self._cached_data: Dict[uuid.UUID, Any] = {}
-        self._load_configs: Dict[uuid.UUID, Optional[LoadConfig]] = {}
+        self._persisted_value_descs: Dict[uuid.UUID, Optional[PersistedData]] = {}
 
         # initialize special values
-        special_value_cls = PythonClass.from_class(SpecialValue)
+        special_value_cls = PythonClass.from_class(NoneType)
         self._not_set_value: Value = Value(
             value_id=NOT_SET_VALUE_ID,
             kiara_id=self._kiara.id,
@@ -103,7 +114,10 @@ class DataRegistry(object):
             pedigree_output_name="__void__",
             data_type_class=special_value_cls,
         )
+        self._not_set_value._data_registry = self
         self._cached_data[NOT_SET_VALUE_ID] = SpecialValue.NOT_SET
+        self._registered_values[NOT_SET_VALUE_ID] = self._not_set_value
+        self._persisted_value_descs[NOT_SET_VALUE_ID] = NONE_PERSISTED_DATA
 
         self._none_value: Value = Value(
             value_id=NONE_VALUE_ID,
@@ -121,7 +135,10 @@ class DataRegistry(object):
             pedigree_output_name="__void__",
             data_type_class=special_value_cls,
         )
+        self._none_value._data_registry = self
         self._cached_data[NONE_VALUE_ID] = SpecialValue.NO_VALUE
+        self._registered_values[NONE_VALUE_ID] = self._none_value
+        self._persisted_value_descs[NONE_VALUE_ID] = NONE_PERSISTED_DATA
 
     @property
     def kiara_id(self) -> uuid.UUID:
@@ -191,13 +208,41 @@ class DataRegistry(object):
     def data_archives(self) -> Mapping[str, DataArchive]:
         return self._data_archives
 
-    def get_archive(self, store_id: Optional[str] = None) -> DataArchive:
-        if store_id is None:
-            store_id = self.default_data_store
-            if store_id is None:
+    def get_archive(
+        self, archive_id: Union[None, uuid.UUID, str] = None
+    ) -> DataArchive:
+
+        if archive_id is None:
+            archive_id = self.default_data_store
+            if archive_id is None:
                 raise Exception("Can't retrieve default data archive, none set (yet).")
 
-        return self._data_archives[store_id]
+        if isinstance(archive_id, uuid.UUID):
+            for archive in self._data_archives.values():
+                if archive.archive_id == archive_id:
+                    return archive
+
+            raise Exception(
+                f"Can't retrieve archive with id '{archive_id}': no archive with that id registered."
+            )
+
+        if archive_id in self._data_archives.keys():
+            return self._data_archives[archive_id]
+        else:
+            try:
+                _archive_id = uuid.UUID(archive_id)
+                for archive in self._data_archives.values():
+                    if archive.archive_id == _archive_id:
+                        return archive
+                    raise Exception(
+                        f"Can't retrieve archive with id '{archive_id}': no archive with that id registered."
+                    )
+            except Exception:
+                pass
+
+        raise Exception(
+            f"Can't retrieve archive with id '{archive_id}': no archive with that id registered."
+        )
 
     def find_store_id_for_value(self, value_id: uuid.UUID) -> Optional[str]:
 
@@ -227,11 +272,16 @@ class DataRegistry(object):
             if hasattr(value_id, "value_id"):
                 _value_id: Optional[uuid.UUID] = value_id.value_id  # type: ignore
             else:
+
                 try:
                     _value_id = uuid.UUID(
                         value_id  # type: ignore
                     )  # this should fail if not string or wrong string format
                 except ValueError:
+                    _value_id = None
+
+                if _value_id is None:
+
                     if not isinstance(value_id, str):
                         raise Exception(
                             f"Can't retrieve value for '{value_id}': invalid type '{type(value_id)}'."
@@ -243,6 +293,7 @@ class DataRegistry(object):
                         )
 
                     ref_type, rest = value_id.split(":", maxsplit=1)
+
                     if ref_type == "value":
                         _value_id = uuid.UUID(rest)
                     elif ref_type == "alias":
@@ -291,8 +342,7 @@ class DataRegistry(object):
         self,
         value: Union[Value, uuid.UUID],
         store_id: Optional[str] = None,
-        skip_if_exists: bool = True,
-    ):
+    ) -> Optional[PersistedData]:
 
         if store_id is None:
             store_id = self.default_data_store
@@ -300,34 +350,36 @@ class DataRegistry(object):
         if isinstance(value, uuid.UUID):
             value = self.get_value(value)
 
-        store: DataStore = self.get_archive(store_id=store_id)  # type: ignore
+        store: DataStore = self.get_archive(archive_id=store_id)  # type: ignore
         if not isinstance(store, DataStore):
             raise Exception(f"Can't store value into store '{store_id}': not writable.")
 
         # make sure all property values are available
         if value.pedigree != ORPHAN:
             for value_id in value.pedigree.inputs.values():
-                self.store_value(value=value_id, store_id=store_id, skip_if_exists=True)
+                self.store_value(value=value_id, store_id=store_id)
 
-        if not store.has_value(value.value_id) or not skip_if_exists:
+        if not store.has_value(value.value_id):
             event = ValuePreStoreEvent.construct(kiara_id=self._kiara.id, value=value)
             self._event_callback(event)
-            load_config = store.store_value(value)
+            persisted_value = store.store_value(value)
             value._is_stored = True
             self._value_archive_lookup_map[value.value_id] = store_id
-            self._load_configs[value.value_id] = load_config
+            self._persisted_value_descs[value.value_id] = persisted_value
             property_values = value.property_values
 
             for property, property_value in property_values.items():
-                self.store_value(
-                    value=property_value, store_id=store_id, skip_if_exists=True
-                )
+                self.store_value(value=property_value, store_id=store_id)
+        else:
+            persisted_value = None
 
         store_event = ValueStoredEvent.construct(kiara_id=self._kiara.id, value=value)
         self._event_callback(store_event)
 
+        return persisted_value
+
     def find_values_for_hash(
-        self, value_hash: int, data_type_name: Optional[str] = None
+        self, value_hash: str, data_type_name: Optional[str] = None
     ) -> Set[Value]:
 
         if data_type_name:
@@ -383,7 +435,7 @@ class DataRegistry(object):
     def register_data(
         self,
         data: Any,
-        schema: Optional[ValueSchema] = None,
+        schema: Union[ValueSchema, str] = None,
         pedigree: Optional[ValuePedigree] = None,
         pedigree_output_name: str = None,
         reuse_existing: bool = True,
@@ -402,12 +454,21 @@ class DataRegistry(object):
             self._registered_values[value.value_id] = value
             self._cached_data[value.value_id] = data
 
+            event = ValueRegisteredEvent(kiara_id=self._kiara.id, value=value)
+            self._event_callback(event)
+
         return value
 
     def _find_existing_value(
         self, data: Any, schema: Optional[ValueSchema]
     ) -> Tuple[
-        Optional[Value], Optional[DataType], Optional[ValueStatus], Optional[int]
+        Optional[Value],
+        DataType,
+        Optional[Any],
+        Union[str, SerializedData],
+        ValueStatus,
+        str,
+        int,
     ]:
 
         if schema is None:
@@ -416,7 +477,20 @@ class DataRegistry(object):
         if isinstance(data, Value):
 
             if data.value_id in self._registered_values.keys():
-                return (data, None, None, None)
+
+                if data.is_set and data.is_serializable:
+                    serialized: Union[str, SerializedData] = data.serialized_data
+                else:
+                    serialized = NO_SERIALIZATION_MARKER
+                return (
+                    data,
+                    data.data_type,
+                    None,
+                    serialized,
+                    data.value_status,
+                    data.value_hash,
+                    data.value_size,
+                )
 
             raise NotImplementedError("Importing values not supported (yet).")
             # self._registered_values[data.value_id] = data
@@ -424,7 +498,20 @@ class DataRegistry(object):
 
         try:
             value = self.get_value(value_id=data)
-            return (value, None, None, None)
+            if value.is_serializable:
+                serialized = value.serialized_data
+            else:
+                serialized = NO_SERIALIZATION_MARKER
+
+            return (
+                value,
+                value.data_type,
+                None,
+                serialized,
+                value.value_status,
+                value.value_hash,
+                value.value_size,
+            )
         except Exception:
             # TODO: differentiate between 'value not found' and other type of errors
             pass
@@ -434,16 +521,9 @@ class DataRegistry(object):
             data_type_name=schema.type, data_type_config=schema.type_config
         )
 
-        if data == SpecialValue.NOT_SET:
-            status = ValueStatus.NOT_SET
-            value_hash = INVALID_HASH_MARKER
-        elif data == SpecialValue.NO_VALUE:
-            status = ValueStatus.NONE
-            value_hash = INVALID_HASH_MARKER
-        else:
-            data, status, value_hash = data_type._pre_examine_data(
-                data=data, schema=schema
-            )
+        data, serialized, status, value_hash, value_size = data_type._pre_examine_data(
+            data=data, schema=schema
+        )
 
         existing_value: Optional[Value] = None
         if value_hash != INVALID_HASH_MARKER:
@@ -469,19 +549,26 @@ class DataRegistry(object):
                             existing_value = orphans[0]
 
         if existing_value is not None:
-            self._load_configs[existing_value.value_id] = None
-            return (existing_value, data_type, status, value_hash)
+            self._persisted_value_descs[existing_value.value_id] = None
+            return (
+                existing_value,
+                data_type,
+                data,
+                serialized,
+                status,
+                value_hash,
+                value_size,
+            )
 
-        return (None, data_type, status, value_hash)
+        return (None, data_type, data, serialized, status, value_hash, value_size)
 
     def _create_value(
         self,
         data: Any,
-        schema: Optional[ValueSchema] = None,
+        schema: Union[None, str, ValueSchema] = None,
         pedigree: Optional[ValuePedigree] = None,
         pedigree_output_name: str = None,
         reuse_existing: bool = True,
-        data_is_value_reference: Optional[bool] = None,
     ) -> Tuple[Value, bool]:
         """Create a new value, or return an existing one that matches the incoming data or reference.
 
@@ -495,23 +582,57 @@ class DataRegistry(object):
 
         if schema is None:
             raise NotImplementedError()
+        elif isinstance(schema, str):
+            schema = ValueSchema(type=schema)
 
         if schema.type not in self._kiara.data_type_names:
             raise Exception(
                 f"Can't register data of type '{schema.type}': type not registered. Available types: {', '.join(self._kiara.data_type_names)}"
             )
 
-        data_type: Optional[DataType] = None
-        status: Optional[ValueStatus] = None
-        value_hash: Optional[int] = None
+        if schema.default in [None, SpecialValue.NO_VALUE, SpecialValue.NOT_SET]:
+            if data in [None, SpecialValue.NO_VALUE]:
+                return (self.NONE_VALUE, False)
+            elif data is SpecialValue.NOT_SET:
+                return (self.NOT_SET_VALUE, False)
+        else:
+            # TODO: allow other value_ids in defaults?
+            if data in [None, SpecialValue.NO_VALUE, SpecialValue.NOT_SET]:
+                if callable(schema.default):
+                    data = schema.default()
+                else:
+                    data = copy.deepcopy(schema.default)
+
+        # data_type: Optional[DataType] = None
+        # status: Optional[ValueStatus] = None
+        # value_hash: Optional[str] = None
 
         if reuse_existing:
-            existing, data_type, status, value_hash = self._find_existing_value(
-                data=data, schema=schema
-            )
-            if existing is not None:
+            (
+                _existing,
+                data_type,
+                data,
+                serialized,
+                status,
+                value_hash,
+                value_size,
+            ) = self._find_existing_value(data=data, schema=schema)
+            if _existing is not None:
                 # TODO: check pedigree
-                return (existing, False)
+                return (_existing, False)
+        else:
+
+            data_type = self._kiara.type_registry.retrieve_data_type(
+                data_type_name=schema.type, data_type_config=schema.type_config
+            )
+
+            (
+                data,
+                serialized,
+                status,
+                value_hash,
+                value_size,
+            ) = data_type._pre_examine_data(data=data, schema=schema)
 
         if pedigree is None:
             pedigree = ORPHAN
@@ -526,28 +647,14 @@ class DataRegistry(object):
             type="value", kiara_id=self._kiara.id, pre_registered=False
         )
 
-        if data_type is None or status is None or value_hash is None:
-            data_type = self._kiara.type_registry.retrieve_data_type(
-                data_type_name=schema.type, data_type_config=schema.type_config
-            )
-
-            if data == SpecialValue.NOT_SET:
-                status = ValueStatus.NOT_SET
-                value_hash = INVALID_HASH_MARKER
-            elif data == SpecialValue.NO_VALUE:
-                status = ValueStatus.NONE
-                value_hash = INVALID_HASH_MARKER
-            else:
-                data, status, value_hash = data_type._pre_examine_data(
-                    data=data, schema=schema
-                )
-
         value, data = data_type.assemble_value(
             value_id=v_id,
             data=data,
             schema=schema,
+            serialized=serialized,
             status=status,
             value_hash=value_hash,
+            value_size=value_size,
             pedigree=pedigree,
             kiara_id=self._kiara.id,
             pedigree_output_name=pedigree_output_name,
@@ -560,108 +667,159 @@ class DataRegistry(object):
 
         return (value, True)
 
-    def retrieve_load_config(self, value_id: uuid.UUID) -> Optional[LoadConfig]:
+    def retrieve_persisted_value_details(self, value_id: uuid.UUID) -> PersistedData:
 
         if (
-            value_id in self._load_configs.keys()
-            and self._load_configs[value_id] is not None
+            value_id in self._persisted_value_descs.keys()
+            and self._persisted_value_descs[value_id] is not None
         ):
-            load_config = self._load_configs[value_id]
+            persisted_details = self._persisted_value_descs[value_id]
+            assert persisted_details is not None
         else:
             # now, the value_store map should contain this value_id
             store_id = self.find_store_id_for_value(value_id=value_id)
             if store_id is None:
-                return None
+                raise Exception(
+                    f"Can't find store for persisted data of value: {value_id}"
+                )
 
             store = self.get_archive(store_id)
             assert value_id in self._registered_values.keys()
             # self.get_value(value_id=value_id)
-            load_config = store.retrieve_load_config(value=value_id)
-            self._load_configs[value_id] = load_config
+            persisted_details = store.retrieve_serialized_value(value=value_id)
+            for c in persisted_details.chunk_id_map.values():
+                c._data_registry = self._kiara.data_registry
+            self._persisted_value_descs[value_id] = persisted_details
 
-        return load_config
+        return persisted_details
 
-    def _retrieve_bytes(
-        self, chunk_id: str, as_bytes: bool = True
-    ) -> Union[str, bytes]:
+    # def _retrieve_bytes(
+    #     self, chunk_id: str, as_link: bool = True
+    # ) -> Union[str, bytes]:
+    #
+    #     # TODO: support multiple stores
+    #     return self.get_archive().retrieve_chunk(chunk_id=chunk_id, as_link=as_link)
 
-        # TODO: support multiple stores
-        return self.get_archive().retrieve_bytes(chunk_id=chunk_id, as_bytes=as_bytes)
+    def retrieve_serialized_value(
+        self, value_id: uuid.UUID
+    ) -> Optional[SerializedData]:
+        """Create a LoadConfig object from the details of the persisted version of this value."""
 
-    def retrieve_value_data(self, value_id: uuid.UUID) -> Any:
-
-        if value_id in self._cached_data.keys():
-            return self._cached_data[value_id]
-
-        load_config = self.retrieve_load_config(value_id=value_id)
-
-        if load_config is None:
-            raise Exception(
-                f"Load config for value '{value_id}' is 'None', this is most likely a bug."
-            )
-
-        data = self._load_data_from_load_config(
-            load_config=load_config, value_id=value_id
-        )
-        self._cached_data[value_id] = data
-
-        return data
-
-    def _provision_bytes(self, load_config: LoadConfig) -> Optional[BytesStructure]:
-
-        provisioning_strategy = load_config.provisioning_strategy
-        if provisioning_strategy == ByteProvisioningStrategy.INLINE:
+        pv = self.retrieve_persisted_value_details(value_id=value_id)
+        if pv is None:
             return None
 
-        bytes_structure_map: Dict[str, List[Union[str, bytes]]] = {}
-        as_bytes = provisioning_strategy == ByteProvisioningStrategy.BYTES
+        return pv
 
-        assert load_config.bytes_map is not None
+    def retrieve_chunk(
+        self,
+        chunk_id: str,
+        archive_id: Optional[uuid.UUID] = None,
+        as_file: Union[None, bool, str] = None,
+        symlink_ok: bool = True,
+    ) -> Union[str, bytes]:
 
-        for field, chunk_ids in load_config.bytes_map.chunk_id_map.items():
-            for chunk_id in chunk_ids:
-                bytes_structure_map.setdefault(field, []).append(
-                    self._retrieve_bytes(chunk_id=chunk_id, as_bytes=as_bytes)
-                )
+        if archive_id is None:
+            raise NotImplementedError()
 
-        bytes_structure = BytesStructure.construct(
-            data_type=load_config.bytes_map.data_type,
-            data_type_config=load_config.bytes_map.data_type_config,
-            chunk_map=bytes_structure_map,
-        )
+        archive = self.get_archive(archive_id)
+        chunk = archive.retrieve_chunk(chunk_id, as_file=as_file, symlink_ok=symlink_ok)
 
-        return bytes_structure
+        return chunk
 
-    def _load_data_from_load_config(
-        self, load_config: LoadConfig, value_id: uuid.UUID
+    def retrieve_value_data(
+        self, value: Union[uuid.UUID, Value], target_profile: Optional[str] = None
     ) -> Any:
 
-        logger.debug("value.load", module=load_config.module_type)
+        if isinstance(value, uuid.UUID):
+            value = self.get_value(value_id=value)
 
-        # TODO: check whether modules and value types are available
-        inputs: Dict[str, Any] = dict(load_config.inputs)
-        if "bytes_structure" in load_config.inputs.keys():
-            bytes_structure = self._provision_bytes(load_config=load_config)
-            inputs["bytes_structure"] = bytes_structure
+        if value.value_id in self._cached_data.keys():
+            return self._cached_data[value.value_id]
 
-        try:
-            job_config = self._kiara.job_registry.prepare_job_config(
-                manifest=load_config, inputs=inputs
+        if value._serialized_data is None:
+            serialized_data: Union[
+                str, SerializedData
+            ] = self.retrieve_persisted_value_details(value_id=value.value_id)
+            value._serialized_data = serialized_data
+        else:
+            serialized_data = value._serialized_data
+
+        if isinstance(serialized_data, str):
+            raise Exception(
+                f"Can't retrieve serialized version of value '{value.value_id}', this is most likely a bug."
             )
-        except JobConfigException:
-            if is_debug():
-                import traceback
 
-                traceback.print_exc()
-            value = self.get_value(value_id=value_id)
-            return UnloadableData(value=value, load_config=load_config)
+        manifest = serialized_data.metadata.deserialize.get("python_object", None)
+        if manifest is None:
+            raise Exception(
+                f"No deserialize operation found for data type: {value.data_type_name}"
+            )
 
-        job_id = self._kiara.job_registry.execute_job(job_config=job_config)
-        result = self._kiara.job_registry.retrieve_result(job_id=job_id)
-        # data = result.get_value_data(load_config.output_name)
-        result_value = result.get_value_obj(field_name=load_config.output_name)
+        module = self._kiara.create_module(manifest=manifest)
+        op = Operation.create_from_module(module=module)
 
-        return result_value.data
+        input_field_match: Optional[str] = None
+        if len(op.inputs_schema) == 1:
+            input_field_match = next(iter(op.inputs_schema.keys()))
+        else:
+            for input_field, schema in op.inputs_schema.items():
+                if schema.type == value.data_type_name:
+                    if input_field_match is not None:
+                        raise Exception(
+                            f"Can't determine input field for deserialization operation '{module.module_type_name}': multiple input fields with type '{input_field_match}'."
+                        )
+                    else:
+                        input_field_match = input_field
+        if input_field_match is None:
+            raise Exception(
+                f"Can't determine input field for deserialization operation '{module.module_type_name}'."
+            )
+
+        result_field_match: Optional[str] = None
+        for result_field, schema in op.outputs_schema.items():
+            if schema.type == "python_object":
+                if result_field_match is not None:
+                    raise Exception(
+                        f"Can't determine result field for deserialization operation '{module.module_type_name}': multiple result fields with type 'python_object'."
+                    )
+                else:
+                    result_field_match = result_field
+        if result_field_match is None:
+            raise Exception(
+                f"Can't determine result field for deserialization operation '{module.module_type_name}'."
+            )
+
+        inputs = {input_field_match: value}
+
+        result = op.run(kiara=self._kiara, inputs=inputs)
+        python_object = result.get_value_data(result_field_match)
+        self._cached_data[value.value_id] = python_object
+
+        return python_object
+
+        # op_type: DeSerializeOperationType = self._kiara.operation_registry.get_operation_type("deserialize")  # type: ignore
+        # ops = op_type.find_deserialzation_operation_for_type_and_profile(
+        #     serialized_data.data_type, serialized_data.serialization_profile
+        # )
+        #
+        # if len(ops) > 1:
+        #     raise Exception("No unique op.")
+        #
+        # if not ops:
+        #     raise Exception(
+        #         f"No deserialize operation found for data type: {value.data_type_name}"
+        #     )
+        #
+        # op = ops[0]
+        # inputs = {"value": serialized_data}
+        #
+        # result = op.run(kiara=self._kiara, inputs=inputs)
+        #
+        # python_object = result.get_value_data("python_object")
+        # self._cached_data[value.value_id] = python_object
+        #
+        # return python_object
 
     def load_values(self, values: Mapping[str, Optional[uuid.UUID]]) -> ValueMap:
 
@@ -710,10 +868,15 @@ class DataRegistry(object):
 
             if input_name not in data.keys():
                 value_data = SpecialValue.NOT_SET
-            elif data[input_name] is None:
+            elif data[input_name] in [
+                None,
+                SpecialValue.NO_VALUE,
+                SpecialValue.NOT_SET,
+            ]:
                 value_data = SpecialValue.NO_VALUE
             else:
                 value_data = data[input_name]
+
             try:
                 value = self.register_data(
                     data=value_data, schema=value_schema, reuse_existing=True
@@ -723,10 +886,13 @@ class DataRegistry(object):
                 # )
                 values[input_name] = value
             except Exception as e:
-                if is_debug():
-                    import traceback
 
-                    traceback.print_exc()
+                log_exception(e)
+
+                msg: Any = str(e)
+                if not msg:
+                    msg = e
+                log_message("invalid.valueset", error_reason=msg, input_name=input_name)
                 failed[input_name] = e
 
         if failed:
@@ -740,7 +906,6 @@ class DataRegistry(object):
                 msg=f"Can't create values instance: {', '.join(msg)}",
                 invalid_values={k: str(v) for k, v in failed.items()},
             )
-
         return ValueMapReadOnly(value_items=values, values_schema=schema)  # type: ignore
 
     def create_renderable(self, **config: Any) -> RenderableType:
@@ -753,7 +918,7 @@ class DataRegistry(object):
         table = create_renderable_from_values(values=all_values, config=config)
         return table
 
-    def render_data(
+    def pretty_print_data(
         self,
         value_id: uuid.UUID,
         target_type="terminal_renderable",
@@ -762,7 +927,7 @@ class DataRegistry(object):
 
         assert isinstance(value_id, uuid.UUID)
 
-        return render_data(
+        return pretty_print_data(
             kiara=self._kiara,
             value_id=value_id,
             target_type=target_type,
