@@ -9,26 +9,10 @@ import abc
 import structlog
 import uuid
 from rich.console import RenderableType
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Dict,
-    Iterable,
-    Mapping,
-    Optional,
-    Set,
-    Tuple,
-    Union,
-)
+from typing import TYPE_CHECKING, Any, Dict, Iterable, Mapping, Optional, Set, Union
 
-from kiara.models.module.persistence import (
-    ByteProvisioningStrategy,
-    BytesAliasStructure,
-    BytesStructure,
-    LoadConfig,
-)
 from kiara.models.runtime_environment import RuntimeEnvironment
-from kiara.models.values.value import Value, ValuePedigree
+from kiara.models.values.value import PersistedData, Value, ValuePedigree
 from kiara.models.values.value_schema import ValueSchema
 from kiara.registries import ARCHIVE_CONFIG_CLS, BaseArchive
 
@@ -48,12 +32,14 @@ class DataArchive(BaseArchive):
 
         super().__init__(archive_id=archive_id, config=config)
 
-        self._env_cache: Dict[str, Dict[int, Mapping[str, Any]]] = {}
+        self._env_cache: Dict[str, Dict[str, Mapping[str, Any]]] = {}
         self._value_cache: Dict[uuid.UUID, Value] = {}
-        self._load_config_cache: Dict[uuid.UUID, LoadConfig] = {}
-        self._value_hash_index: Dict[int, Set[uuid.UUID]] = {}
+        self._persisted_value_cache: Dict[uuid.UUID, PersistedData] = {}
+        self._value_hash_index: Dict[str, Set[uuid.UUID]] = {}
 
-    def retrieve_load_config(self, value: Union[uuid.UUID, Value]) -> LoadConfig:
+    def retrieve_serialized_value(
+        self, value: Union[uuid.UUID, Value]
+    ) -> PersistedData:
 
         if isinstance(value, Value):
             value_id: uuid.UUID = value.value_id
@@ -62,20 +48,20 @@ class DataArchive(BaseArchive):
             value_id = value
             _value = None
 
-        if value_id in self._load_config_cache.keys():
-            return self._load_config_cache[value_id]
+        if value_id in self._persisted_value_cache.keys():
+            return self._persisted_value_cache[value_id]
 
         if _value is None:
             _value = self.retrieve_value(value_id)
 
         assert _value is not None
 
-        load_config = self._retrieve_load_config(value=_value)
-        self._load_config_cache[_value.value_id] = load_config
-        return load_config
+        persisted_value = self._retrieve_serialized_value(value=_value)
+        self._persisted_value_cache[_value.value_id] = persisted_value
+        return persisted_value
 
     @abc.abstractmethod
-    def _retrieve_load_config(self, value: Value) -> LoadConfig:
+    def _retrieve_serialized_value(self, value: Value) -> PersistedData:
         pass
 
     def retrieve_value(self, value_id: uuid.UUID) -> Value:
@@ -92,7 +78,6 @@ class DataArchive(BaseArchive):
         #     )
 
         pedigree = ValuePedigree(**value_data["pedigree"])
-
         value = Value(
             value_id=value_data["value_id"],
             kiara_id=self.kiara_context.id,
@@ -139,7 +124,7 @@ class DataArchive(BaseArchive):
         return value_id in self._retrieve_all_value_ids()
 
     def retrieve_environment_details(
-        self, env_type: str, env_hash: int
+        self, env_type: str, env_hash: str
     ) -> Mapping[str, Any]:
         """Retrieve the environment details with the specified type and hash.
 
@@ -157,13 +142,13 @@ class DataArchive(BaseArchive):
 
     @abc.abstractmethod
     def _retrieve_environment_details(
-        self, env_type: str, env_hash: int
+        self, env_type: str, env_hash: str
     ) -> Mapping[str, Any]:
         pass
 
     def find_values_with_hash(
         self,
-        value_hash: int,
+        value_hash: str,
         value_size: Optional[int] = None,
         data_type_name: Optional[str] = None,
     ) -> Set[uuid.UUID]:
@@ -190,7 +175,7 @@ class DataArchive(BaseArchive):
     @abc.abstractmethod
     def _find_values_with_hash(
         self,
-        value_hash: int,
+        value_hash: str,
         value_size: Optional[int] = None,
         data_type_name: Optional[str] = None,
     ) -> Optional[Set[uuid.UUID]]:
@@ -211,7 +196,12 @@ class DataArchive(BaseArchive):
         pass
 
     @abc.abstractmethod
-    def retrieve_bytes(self, chunk_id: str, as_bytes: bool = True) -> Union[bytes, str]:
+    def retrieve_chunk(
+        self,
+        chunk_id: str,
+        as_file: Union[bool, str, None] = None,
+        symlink_ok: bool = True,
+    ) -> Union[bytes, str]:
         pass
 
     # def retrieve_job_record(self, inputs_manifest: InputsManifest) -> Optional[JobRecord]:
@@ -232,7 +222,7 @@ class DataStore(DataArchive):
         return True
 
     @abc.abstractmethod
-    def store_value(self, value: Value) -> LoadConfig:
+    def store_value(self, value: Value) -> PersistedData:
         """ "Store the value, its data and metadata into the store.
 
         Arguments:
@@ -244,12 +234,12 @@ class DataStore(DataArchive):
 
 
 class BaseDataStore(DataStore):
-    @abc.abstractmethod
-    def _persist_bytes(self, bytes_structure: BytesStructure) -> BytesAliasStructure:
-        pass
+    # @abc.abstractmethod
+    # def _persist_bytes(self, bytes_structure: BytesStructure) -> BytesAliasStructure:
+    #     pass
 
     @abc.abstractmethod
-    def _persist_load_config(self, value: Value, load_config: LoadConfig):
+    def _persist_stored_value_info(self, value: Value, persisted_value: PersistedData):
         pass
 
     @abc.abstractmethod
@@ -257,9 +247,7 @@ class BaseDataStore(DataStore):
         pass
 
     @abc.abstractmethod
-    def _persist_value_data(
-        self, value: Value
-    ) -> Tuple[LoadConfig, Optional[BytesStructure]]:
+    def _persist_value_data(self, value: Value) -> PersistedData:
         pass
 
     @abc.abstractmethod
@@ -272,7 +260,7 @@ class BaseDataStore(DataStore):
 
     @abc.abstractmethod
     def _persist_environment_details(
-        self, env_type: str, env_hash: int, env_data: Mapping[str, Any]
+        self, env_type: str, env_hash: str, env_data: Mapping[str, Any]
     ):
         pass
 
@@ -280,7 +268,7 @@ class BaseDataStore(DataStore):
     def _persist_destiny_backlinks(self, value: Value):
         pass
 
-    def store_value(self, value: Value) -> LoadConfig:
+    def store_value(self, value: Value) -> PersistedData:
 
         logger.debug(
             "store.value",
@@ -295,14 +283,14 @@ class BaseDataStore(DataStore):
             if cached is not None:
                 continue
 
-            env = self.kiara_context.environment_registry.get_environment_for_hash(
+            env = self.kiara_context.environment_registry.get_environment_for_cid(
                 env_hash
             )
             self.persist_environment(env)
 
         # save the value data and metadata
-        load_config = self._persist_value(value)
-        self._load_config_cache[value.value_id] = load_config
+        persisted_value = self._persist_value(value)
+        self._persisted_value_cache[value.value_id] = persisted_value
         self._value_cache[value.value_id] = value
         self._value_hash_index.setdefault(value.value_hash, set()).add(value.value_id)
 
@@ -310,44 +298,29 @@ class BaseDataStore(DataStore):
         # then, make sure the manifest is persisted
         self._persist_value_pedigree(value=value)
 
-        return load_config
+        return persisted_value
 
-    def _persist_value(self, value: Value) -> LoadConfig:
+    def _persist_value(self, value: Value) -> PersistedData:
 
         # TODO: check if value id is already persisted?
-        load_config, bytes_structure = self._persist_value_data(value=value)
-
-        if not load_config:
+        persisted_value_info: PersistedData = self._persist_value_data(value=value)
+        if not persisted_value_info:
             raise Exception(
-                "Can't write load config, no load config returned when persisting value."
+                "Can't write persisted value info, no load config returned when persisting value."
             )
-        if not isinstance(load_config, LoadConfig):
+        if not isinstance(persisted_value_info, PersistedData):
             raise Exception(
-                f"Can't write load config, invalid result type '{type(load_config)}' when persisting value."
+                f"Can't write persisted value info, invalid result type '{type(persisted_value_info)}' when persisting value."
             )
 
-        if (
-            load_config.provisioning_strategy != ByteProvisioningStrategy.INLINE
-            and not bytes_structure
-        ):
-            raise Exception(
-                "Can't write load config, no bytes structure returned when persisting value."
-            )
-
-        if bytes_structure and not isinstance(bytes_structure, BytesStructure):
-            raise Exception(
-                f"Can't write load config, invalid result bytes structure type '{type(bytes_structure)}' when persisting value."
-            )
-
-        if bytes_structure:
-            alias_structure = self._persist_bytes(bytes_structure=bytes_structure)
-            load_config.bytes_map = alias_structure
-        self._persist_load_config(value=value, load_config=load_config)
+        self._persist_stored_value_info(
+            value=value, persisted_value=persisted_value_info
+        )
         self._persist_value_details(value=value)
         if value.destiny_backlinks:
             self._persist_destiny_backlinks(value=value)
 
-        return load_config
+        return persisted_value_info
 
     def persist_environment(self, environment: RuntimeEnvironment):
         """Persist the specified environment.
@@ -357,7 +330,7 @@ class BaseDataStore(DataStore):
         """
 
         env_type = environment.get_environment_type_name()
-        env_hash = environment.model_data_hash
+        env_hash = str(environment.instance_cid)
 
         env = self._env_cache.get(env_type, {}).get(env_hash, None)
         if env is not None:
