@@ -4,43 +4,29 @@
 #  Copyright (c) 2021, Markus Binsteiner
 #
 #  Mozilla Public License, version 2.0 (see LICENSE or https://www.mozilla.org/en-US/MPL/2.0/)
-
 import orjson
 import structlog
 import uuid
 from enum import Enum
-from hashfs import HashFS
 from io import BytesIO
 from pathlib import Path
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Dict,
-    Iterable,
-    List,
-    Mapping,
-    Optional,
-    Set,
-    Tuple,
-    Union,
-)
+from typing import TYPE_CHECKING, Any, Iterable, Mapping, Optional, Set, Union
 
 from kiara.models.module.jobs import JobRecord
 from kiara.models.module.manifest import InputsManifest
-from kiara.models.module.persistence import (
-    BytesAliasStructure,
-    BytesStructure,
-    LoadConfig,
-)
-from kiara.models.values.value import Value
-from kiara.operations.included_core_operations.persistence import (
-    PersistValueOperationType,
+from kiara.models.values.value import (
+    SERIALIZE_TYPES,
+    PersistedData,
+    SerializedChunkIDs,
+    SerializedData,
+    Value,
 )
 from kiara.registries import FileSystemArchiveConfig
 from kiara.registries.data.data_store import BaseDataStore, DataArchive
 from kiara.registries.ids import ID_REGISTRY
 from kiara.registries.jobs import JobArchive
-from kiara.utils import is_debug, log_message, orjson_dumps
+from kiara.utils import log_message, orjson_dumps
+from kiara.utils.hashfs import HashAddress, HashFS
 
 if TYPE_CHECKING:
     pass
@@ -95,7 +81,7 @@ class FileSystemDataArchive(DataArchive, JobArchive):
         if self._base_path is not None:
             return self._base_path
 
-        self._base_path = Path(self.config.base_path) / str(self.archive_id)
+        self._base_path = Path(self.config.archive_path).absolute()
         self._base_path.mkdir(parents=True, exist_ok=True)
         return self._base_path
 
@@ -111,7 +97,7 @@ class FileSystemDataArchive(DataArchive, JobArchive):
 
         if self._hashfs is None:
             self._hashfs = HashFS(
-                self.hash_fs_path,
+                self.hash_fs_path.as_posix(),
                 depth=DEFAULT_HASHFS_DEPTH,
                 width=DEFAULT_HASHFS_WIDTH,
                 algorithm=DEFAULT_HASH_FS_ALGORITHM,
@@ -136,7 +122,7 @@ class FileSystemDataArchive(DataArchive, JobArchive):
         return result
 
     def _retrieve_environment_details(
-        self, env_type: str, env_hash: int
+        self, env_type: str, env_hash: str
     ) -> Mapping[str, Any]:
 
         base_path = self.get_path(entity_type=EntityType.ENVIRONMENT)
@@ -155,12 +141,12 @@ class FileSystemDataArchive(DataArchive, JobArchive):
     ) -> Optional[JobRecord]:
 
         return self._retrieve_job_record(
-            manifest_hash=inputs_manifest.manifest_hash,
+            manifest_hash=str(inputs_manifest.instance_cid),
             jobs_hash=inputs_manifest.job_hash,
         )
 
     def _retrieve_job_record(
-        self, manifest_hash: int, jobs_hash: int
+        self, manifest_hash: str, jobs_hash: str
     ) -> Optional[JobRecord]:
 
         base_path = self.get_path(entity_type=EntityType.MANIFEST)
@@ -178,7 +164,7 @@ class FileSystemDataArchive(DataArchive, JobArchive):
 
         manifest_data = orjson.loads(manifest_file.read_text())
 
-        job_folder = manifest_folder / str(jobs_hash)
+        job_folder = manifest_folder / jobs_hash
 
         if not job_folder.exists():
             return None
@@ -216,7 +202,7 @@ class FileSystemDataArchive(DataArchive, JobArchive):
 
     def _find_values_with_hash(
         self,
-        value_hash: int,
+        value_hash: str,
         value_size: Optional[int] = None,
         data_type_name: Optional[str] = None,
     ) -> Set[uuid.UUID]:
@@ -307,23 +293,31 @@ class FileSystemDataArchive(DataArchive, JobArchive):
         value_data = orjson.loads(base_path.read_text())
         return value_data
 
-    def _retrieve_load_config(self, value: Value) -> LoadConfig:
+    def _retrieve_serialized_value(self, value: Value) -> PersistedData:
 
         base_path = self.get_path(entity_type=EntityType.VALUE_DATA)
         data_dir = base_path / value.data_type_name / str(value.value_hash)
 
-        load_config_file = data_dir / ".load_config.json"
-        data = orjson.loads(load_config_file.read_text())
+        serialized_value_file = data_dir / ".serialized_value.json"
+        data = orjson.loads(serialized_value_file.read_text())
 
-        return LoadConfig(**data)
+        return PersistedData(**data)
 
-    def retrieve_bytes(self, chunk_id: str, as_bytes: bool = True) -> Union[bytes, str]:
+    def retrieve_chunk(
+        self,
+        chunk_id: str,
+        as_file: Union[bool, str, None] = None,
+        symlink_ok: bool = True,
+    ) -> Union[bytes, str]:
 
         addr = self.hashfs.get(chunk_id)
-        if as_bytes:
+
+        if as_file in (None, True):
+            return addr.abspath
+        elif as_file is False:
             return Path(addr.abspath).read_bytes()
         else:
-            return addr.abspath
+            raise NotImplementedError()
 
 
 class FilesystemDataStore(FileSystemDataArchive, BaseDataStore):
@@ -332,7 +326,7 @@ class FilesystemDataStore(FileSystemDataArchive, BaseDataStore):
     _archive_type_name = "filesystem_data_store"
 
     def _persist_environment_details(
-        self, env_type: str, env_hash: int, env_data: Mapping[str, Any]
+        self, env_type: str, env_hash: str, env_data: Mapping[str, Any]
     ):
 
         base_path = self.get_path(entity_type=EntityType.ENVIRONMENT)
@@ -341,13 +335,13 @@ class FilesystemDataStore(FileSystemDataArchive, BaseDataStore):
         if not env_details_file.exists():
             env_details_file.write_text(orjson_dumps(env_data))
 
-    def _persist_load_config(self, value: Value, load_config: LoadConfig):
+    def _persist_stored_value_info(self, value: Value, persisted_value: PersistedData):
 
         working_dir = self.get_path(entity_type=EntityType.VALUE_DATA)
         data_dir = working_dir / value.data_type_name / str(value.value_hash)
-        load_config_file = data_dir / ".load_config.json"
+        sv_file = data_dir / ".serialized_value.json"
         data_dir.mkdir(exist_ok=True, parents=True)
-        load_config_file.write_text(load_config.json())
+        sv_file.write_text(persisted_value.json())
 
     def _persist_value_details(self, value: Value):
 
@@ -383,69 +377,61 @@ class FilesystemDataStore(FileSystemDataArchive, BaseDataStore):
 
             destiny_file.symlink_to(value_file)
 
-    def _persist_bytes(self, bytes_structure: BytesStructure) -> BytesAliasStructure:
+    def _persist_value_data(self, value: Value) -> PersistedData:
 
-        bytes_alias_map: Dict[str, List[str]] = {}
+        serialized_value: SerializedData = value.serialized_data
 
-        for key, bytes_list in bytes_structure.chunk_map.items():
+        chunk_id_map = {}
+        for key in serialized_value.get_keys():
 
-            if is_debug():
-                assert not isinstance(bytes_list, (bytes, str))
+            data_model = serialized_value.get_serialized_data(key)
 
-            for chunk in bytes_list:
-                if isinstance(chunk, str):
-                    addr = self.hashfs.put(chunk)
-                elif isinstance(chunk, bytes):
-                    _chunk = BytesIO(chunk)
-                    addr = self.hashfs.put(_chunk)
-                else:
-                    addr = self.hashfs.put(chunk)
-                    # raise Exception(
-                    #     f"Can't persist chunk: invalid type '{type(chunk)}'"
-                    # )
-                bytes_alias_map.setdefault(key, []).append(addr.id)
+            if data_model.type == "chunk":  # type: ignore
+                chunks: Iterable[Union[str, BytesIO]] = [BytesIO(data_model.chunk)]  # type: ignore
+            elif data_model.type == "chunks":  # type: ignore
+                chunks = (BytesIO(c) for c in data_model.chunks)  # type: ignore
+            elif data_model.type == "file":  # type: ignore
+                chunks = [data_model.file]  # type: ignore
+            elif data_model.type == "files":  # type: ignore
+                chunks = data_model.files  # type: ignore
+            elif data_model.type == "inline-json":  # type: ignore
+                chunks = [BytesIO(data_model.as_json())]  # type: ignore
+            else:
+                raise Exception(
+                    f"Invalid serialized data type: {type(data_model)}. Available types: {', '.join(SERIALIZE_TYPES)}"
+                )
 
-        alias_structure = BytesAliasStructure.construct(
-            data_type=bytes_structure.data_type,
-            data_type_config=bytes_structure.data_type_config,
-            chunk_id_map=bytes_alias_map,
-        )
-        return alias_structure
+            chunk_ids = []
+            for item in zip(serialized_value.get_cids_for_key(key), chunks):
+                cid = item[0]
+                _chunk = item[1]
+                addr: HashAddress = self.hashfs.put_with_precomputed_hash(
+                    _chunk, str(cid)
+                )
+                chunk_ids.append(addr.id)
 
-    def _persist_value_data(
-        self, value: Value
-    ) -> Tuple[LoadConfig, Optional[BytesStructure]]:
-
-        persist_op_type = self.kiara_context.operation_registry.operation_types.get(
-            "persist_value", None
-        )
-        if persist_op_type is None:
-            raise Exception(
-                "Can't persist value, 'persist_value' operation type not available."
+            scids = SerializedChunkIDs(
+                chunk_id_list=chunk_ids,
+                archive_id=self.archive_id,
+                size=data_model.get_size(),
             )
+            scids._data_registry = self.kiara_context.data_registry
+            chunk_id_map[key] = scids
 
-        op_type: PersistValueOperationType = self._kiara.operation_registry.get_operation_type("persist_value")  # type: ignore
-        op = op_type.get_operation_for_data_type(value.value_schema.type)
-
-        working_dir = self.get_path(entity_type=EntityType.VALUE_DATA)
-        data_dir = working_dir / value.data_type_name / str(value.value_hash)
-
-        result = op.run(
-            kiara=self.kiara_context,
-            inputs={
-                "value": value,
-                "persistence_config": {"temp_dir": data_dir.as_posix()},
-            },
+        pers_value = PersistedData(
+            archive_id=self.archive_id,
+            chunk_id_map=chunk_id_map,
+            data_type=serialized_value.data_type,
+            data_type_config=serialized_value.data_type_config,
+            serialization_profile=serialized_value.serialization_profile,
+            metadata=serialized_value.metadata,
         )
 
-        load_config: LoadConfig = result.get_value_data("load_config")
-        bytes_structure: BytesStructure = result.get_value_data("bytes_structure")
-
-        return load_config, bytes_structure
+        return pers_value
 
     def _persist_value_pedigree(self, value: Value):
 
-        manifest_hash = value.pedigree.manifest_hash
+        manifest_hash = value.pedigree.instance_cid
         jobs_hash = value.pedigree.job_hash
 
         base_path = self.get_path(entity_type=EntityType.MANIFEST)
