@@ -13,10 +13,12 @@ from stevedore import ExtensionManager
 from types import ModuleType
 from typing import (
     TYPE_CHECKING,
+    Any,
     Callable,
     Dict,
     Iterable,
     List,
+    Mapping,
     Optional,
     Tuple,
     Type,
@@ -29,15 +31,16 @@ from kiara.utils import (
     _import_modules_recursively,
     camel_case_to_snake_case,
     is_debug,
+    log_exception,
     log_message,
 )
 
 if TYPE_CHECKING:
     from kiara.modules import KiaraModule
     from kiara.operations import OperationType
-    from kiara.models.values.value_metadata import ValueMetadata
     from kiara.data_types import DataType
     from kiara.registries import KiaraArchive
+    from kiara.models import KiaraModel
 
 import logging
 import structlog
@@ -94,10 +97,15 @@ def find_subclasses_under(
     if hasattr(sys, "frozen"):
         raise NotImplementedError("Pyinstaller bundling not supported yet.")
 
-    if isinstance(python_module, str):
-        python_module = importlib.import_module(python_module)
+    try:
+        if isinstance(python_module, str):
+            python_module = importlib.import_module(python_module)
 
-    _import_modules_recursively(python_module)
+        _import_modules_recursively(python_module)
+    except Exception as e:
+        log_exception(e)
+        log_message("ignore.python_module", module=str(python_module), reason=str(e))
+        return []
 
     subclasses: Iterable[Type[SUBCLASS_TYPE]] = _get_all_subclasses(base_class)
 
@@ -120,6 +128,7 @@ def _process_subclass(
     type_id_no_attach: bool,
     attach_python_metadata: Union[bool, str] = False,
     ignore_abstract_classes: bool = True,
+    ignore_modules_with_null_module_name: bool = True,
 ) -> Optional[str]:
     """Process subclasses of a base class that live under a module (recursively).
 
@@ -131,6 +140,7 @@ def _process_subclass(
         type_id_func: a function to take the found class as input, and returns a string representing the id of the class. By default, the module path + "." + class name (snake-case) is used (minus the string 'kiara_modules.<project_name>'', if it exists at the beginning
         type_id_no_attach: in case you want to use the type_id_key to set the id, but don't want it attached to classes that don't have it, set this to true. In most cases, you won't need this option
         attach_python_metadata: whether to attach a [PythonClass][kiara.models.python_class.PythonClass] metadata model to the class. By default, '_python_class' is used as attribute name if this argument is 'True', If this argument is a string, that will be used as name instead.
+        ignore_modules_with_null_module_name: ignore modules that have their '_module_type_name' attribute set to 'None', this is mostly useful to filter out base classes
 
     Returns:
         the type id
@@ -152,6 +162,14 @@ def _process_subclass(
 
         if hasattr(sub_class, type_id_key):
             type_id = getattr(sub_class, type_id_key)
+            if type_id is None and ignore_modules_with_null_module_name:
+                log_message(
+                    "ignore.subclass",
+                    sub_class=sub_class,
+                    base_class=base_class,
+                    reason="'_module_type_name' subclass is set to 'None'",
+                )
+                return None
             if not type_id and not is_abstract:
                 raise Exception(
                     f"Class attribute '{type_id_key}' is 'None' for class '{sub_class.__name__}', this is not allowed."
@@ -223,6 +241,7 @@ def load_all_subclasses_for_entry_point(
 
     result_entrypoints: Dict[str, Type[SUBCLASS_TYPE]] = {}
     result_dynamic: Dict[str, Type[SUBCLASS_TYPE]] = {}
+
     for plugin in mgr:
         name = plugin.name
 
@@ -287,7 +306,7 @@ def load_all_subclasses_for_entry_point(
 
                 if type_id in result_dynamic.keys():
                     raise Exception(
-                        f"Duplicate item name for type {entry_point_name}: {type_id}"
+                        f"Duplicate type id '{type_id}' for type {entry_point_name}: {result_dynamic[type_id]} -- {sub_class}"
                     )
                 result_dynamic[type_id] = sub_class
 
@@ -339,23 +358,40 @@ def find_all_kiara_modules() -> Dict[str, Type["KiaraModule"]]:
     return result
 
 
-def find_all_value_metadata_models() -> Dict[str, Type["ValueMetadata"]]:
+def find_all_kiara_model_classes() -> Dict[str, Type["KiaraModel"]]:
     """Find all [KiaraModule][kiara.module.KiaraModule] subclasses via package entry points.
 
     TODO
     """
 
-    from kiara.models.values.value_metadata import ValueMetadata
+    from kiara.models import KiaraModel
 
     return load_all_subclasses_for_entry_point(
-        entry_point_name="kiara.value_metadata",
-        base_class=ValueMetadata,  # type: ignore
-        type_id_key="_metadata_key",
+        entry_point_name="kiara.model_classes",
+        base_class=KiaraModel,  # type: ignore
+        type_id_key="_kiara_model_id",
         type_id_func=_cls_name_id_func,
         attach_python_metadata=False,
     )
 
 
+# def find_all_value_metadata_models() -> Dict[str, Type["ValueMetadata"]]:
+#     """Find all [KiaraModule][kiara.module.KiaraModule] subclasses via package entry points.
+#
+#     TODO
+#     """
+#
+#     from kiara.models.values.value_metadata import ValueMetadata
+#
+#     return load_all_subclasses_for_entry_point(
+#         entry_point_name="kiara.metadata_models",
+#         base_class=ValueMetadata,  # type: ignore
+#         type_id_key="_metadata_key",
+#         type_id_func=_cls_name_id_func,
+#         attach_python_metadata=False,
+#     )
+
+
 def find_all_archive_types() -> Dict[str, Type["KiaraArchive"]]:
     """Find all [KiaraArchive][kiara.registries.KiaraArchive] subclasses via package entry points."""
 
@@ -418,17 +454,33 @@ def find_kiara_modules_under(
     )
 
 
-def find_value_metadata_models_under(
+def find_kiara_model_classes_under(
     module: Union[str, ModuleType]
-) -> List[Type["ValueMetadata"]]:
+) -> List[Type["KiaraModel"]]:
 
-    from kiara.models.values.value_metadata import ValueMetadata
+    from kiara.models import KiaraModel
 
-    return find_subclasses_under(
-        base_class=ValueMetadata,  # type: ignore
+    result = find_subclasses_under(
+        base_class=KiaraModel,  # type: ignore
         python_module=module,
     )
 
+    return result
+
+
+# def find_value_metadata_models_under(
+#     module: Union[str, ModuleType]
+# ) -> List[Type["ValueMetadata"]]:
+#
+#     from kiara.models.values.value_metadata import ValueMetadata
+#
+#     result = find_subclasses_under(
+#         base_class=ValueMetadata,  # type: ignore
+#         python_module=module,
+#     )
+#
+#     return result
+
 
 def find_data_types_under(module: Union[str, ModuleType]) -> List[Type["DataType"]]:
 
@@ -473,7 +525,7 @@ def find_pipeline_base_path_for_module(module: Union[str, ModuleType]) -> Option
 
 def find_all_kiara_pipeline_paths(
     skip_errors: bool = False,
-) -> List[str]:
+) -> Dict[str, Optional[Mapping[str, Any]]]:
 
     import logging
 
@@ -492,7 +544,7 @@ def find_all_kiara_pipeline_paths(
         namespace="kiara.pipelines", invoke_on_load=False, propagate_map_exceptions=True
     )
 
-    paths: List[str] = []
+    paths: Dict[str, Optional[Mapping[str, Any]]] = {}
     # TODO: make sure we load 'core' first?
     for plugin in mgr:
 
@@ -509,11 +561,32 @@ def find_all_kiara_pipeline_paths(
                 else:
                     func = plugin.plugin[0]
                     args = plugin.plugin[1:]
-                result = func(*args)
+
+                f_args = []
+                metadata: Optional[Mapping[str, Any]] = None
+                if len(args) >= 1:
+                    f_args.append(args[0])
+                if len(args) >= 2:
+                    metadata = args[1]
+                    assert isinstance(metadata, Mapping)
+                if len(args) > 3:
+                    logger.debug(
+                        "ignore.pipeline_lookup_arguments",
+                        reason="more than 2 arguments provided",
+                        surplus_args=args[2:],
+                        path=f_args[0],
+                    )
+
+                result = func(f_args[0])
+                if not result:
+                    continue
                 if isinstance(result, str):
-                    paths.append(result)
+                    paths[result] = metadata
                 else:
-                    paths.extend(result)
+                    for path in paths:
+                        assert path not in paths.keys()
+                        paths[path] = metadata
+
             except Exception as e:
                 if is_debug():
                     import traceback
@@ -539,6 +612,35 @@ def find_all_kiara_pipeline_paths(
     return paths
 
 
+def find_all_cli_subcommands():
+
+    entry_point_name = "kiara.cli_subcommands"
+    log2 = logging.getLogger("stevedore")
+    out_hdlr = logging.StreamHandler(sys.stdout)
+    out_hdlr.setFormatter(
+        logging.Formatter(
+            f"{entry_point_name} plugin search message/error -> %(message)s"
+        )
+    )
+    out_hdlr.setLevel(logging.INFO)
+    log2.addHandler(out_hdlr)
+    if is_debug():
+        log2.setLevel(logging.DEBUG)
+    else:
+        out_hdlr.setLevel(logging.INFO)
+        log2.setLevel(logging.INFO)
+
+    log_message("events.loading.entry_points", entry_point_name=entry_point_name)
+
+    mgr = ExtensionManager(
+        namespace=entry_point_name,
+        invoke_on_load=False,
+        propagate_map_exceptions=True,
+    )
+
+    return [plugin.plugin for plugin in mgr]
+
+
 # def _find_pipeline_folders_using_callable(
 #     func: Union[Callable, Tuple]
 # ) -> Tuple[Optional[str], str]:
