@@ -25,11 +25,12 @@ from typing import (
 )
 
 from kiara.defaults import SpecialValue
-from kiara.models.values.value import ORPHAN, Value
+from kiara.models.values.value import ORPHAN, Value, ValueMap
 from kiara.utils import dict_from_cli_args, orjson_dumps
 
 if TYPE_CHECKING:
     from pyarrow import Table as ArrowTable
+    from sqlalchemy.engine import Engine
 
     from kiara.models.values.value_schema import ValueSchema
 
@@ -248,9 +249,10 @@ class TabularWrap(ABC):
         rows_tail: Optional[int] = None,
         max_row_height: Optional[int] = None,
         max_cell_length: Optional[int] = None,
+        show_table_header: bool = True,
     ) -> RenderableType:
 
-        rich_table = RichTable(box=box.SIMPLE)
+        rich_table = RichTable(box=box.SIMPLE, show_header=show_table_header)
         for cn in self.retrieve_column_names():
             rich_table.add_column(cn)
 
@@ -413,6 +415,72 @@ class DictTabularWrap(TabularWrap):
         return DictTabularWrap(result)
 
 
+class SqliteTabularWrap(TabularWrap):
+    def __init__(self, engine: "Engine", table_name: str):
+        self._engine: Engine = engine
+        self._table_name: str = table_name
+        super().__init__()
+
+    def retrieve_number_of_rows(self) -> int:
+
+        from sqlalchemy import text
+
+        with self._engine.connect() as con:
+            result = con.execute(text(f"SELECT count(*) from {self._table_name}"))
+            num_rows = result.fetchone()[0]
+
+        return num_rows
+
+    def retrieve_column_names(self) -> Iterable[str]:
+
+        from sqlalchemy import inspect
+
+        engine = self._engine
+        inspector = inspect(engine)
+        columns = inspector.get_columns(self._table_name)
+        result = [column["name"] for column in columns]
+        return result
+
+    def slice(self, offset: int = 0, length: Optional[int] = None) -> "TabularWrap":
+
+        from sqlalchemy import text
+
+        query = f"SELECT * FROM {self._table_name}"
+        if length:
+            query = f"{query} LIMIT {length}"
+        else:
+            query = f"{query} LIMIT {self.num_rows}"
+        if offset > 0:
+            query = f"{query} OFFSET {offset}"
+        with self._engine.connect() as con:
+            result = con.execute(text(query))
+            result_dict: Dict[str, List[Any]] = {}
+            for cn in self.column_names:
+                result_dict[cn] = []
+            for r in result:
+                for i, cn in enumerate(self.column_names):
+                    result_dict[cn].append(r[i])
+
+        return DictTabularWrap(result_dict)
+
+    def to_pydict(self) -> Mapping:
+
+        from sqlalchemy import text
+
+        query = f"SELECT * FROM {self._table_name}"
+
+        with self._engine.connect() as con:
+            result = con.execute(text(query))
+            result_dict: Dict[str, List[Any]] = {}
+            for cn in self.column_names:
+                result_dict[cn] = []
+            for r in result:
+                for i, cn in enumerate(self.column_names):
+                    result_dict[cn].append(r[i])
+
+        return result_dict
+
+
 def create_table_from_base_model_cls(model_cls: Type[BaseModel]):
 
     table = RichTable(box=box.SIMPLE, show_lines=True)
@@ -492,12 +560,12 @@ def create_table_from_field_schemas(
     _show_header: bool = False,
     _constants: Optional[Mapping[str, Any]] = None,
     **fields: "ValueSchema",
-):
+) -> RichTable:
 
     table = RichTable(box=box.SIMPLE, show_header=_show_header)
-    table.add_column("Field name", style="i")
-    table.add_column("Type")
-    table.add_column("Description")
+    table.add_column("field name", style="i")
+    table.add_column("type")
+    table.add_column("description")
 
     if _add_required:
         table.add_column("Required")
@@ -545,6 +613,77 @@ def create_table_from_field_schemas(
     return table
 
 
+def create_value_map_status_renderable(
+    inputs: ValueMap, render_config: Optional[Mapping[str, Any]] = None
+) -> RichTable:
+
+    if render_config is None:
+        render_config = {}
+
+    show_required: bool = render_config.get("show_required", True)
+    show_default: bool = render_config.get("show_default", True)
+
+    table = RichTable(box=box.SIMPLE, show_header=True)
+    table.add_column("field name", style="i")
+    table.add_column("status", style="b")
+    table.add_column("type")
+    table.add_column("description")
+
+    if show_required:
+        table.add_column("required")
+
+    if show_default:
+        table.add_column("default")
+
+    invalid = inputs.check_invalid()
+
+    for field_name, value in inputs.items():
+
+        row: List[RenderableType] = [field_name]
+
+        if field_name in invalid.keys():
+            row.append(f"[red]{invalid[field_name]}[/red]")
+        else:
+            row.append("[green]valid[/green]")
+
+        value_schema = inputs.values_schema[field_name]
+
+        row.extend([value_schema.type, value_schema.doc.description])
+
+        if show_required:
+            req = value_schema.is_required()
+            if not req:
+                req_str = "no"
+            else:
+                if value_schema.default in [
+                    None,
+                    SpecialValue.NO_VALUE,
+                    SpecialValue.NOT_SET,
+                ]:
+                    req_str = "[b]yes[b]"
+                else:
+                    req_str = "no"
+            row.append(req_str)
+
+        if show_default:
+            default = value_schema.default
+            if callable(default):
+                default_val = default()
+            else:
+                default_val = default
+
+            if default_val in [None, SpecialValue.NOT_SET, SpecialValue.NO_VALUE]:
+                default_str = ""
+            else:
+                default_str = str(default_val)
+
+            row.append(default_str)
+
+        table.add_row(*row)
+
+    return table
+
+
 def create_table_from_model_object(
     model: BaseModel,
     render_config: Optional[Mapping[str, Any]] = None,
@@ -645,7 +784,7 @@ def create_renderable_from_values(
     show_pedigree = config.get("show_pedigree", False)
     show_data = config.get("show_data", False)
     show_hash = config.get("show_hash", True)
-    show_load_config = config.get("show_load_config", False)
+    # show_load_config = config.get("show_load_config", False)
 
     table = RichTable(show_lines=True, box=box.MINIMAL_DOUBLE_HEAD)
     table.add_column("value_id", "i")
@@ -657,8 +796,6 @@ def create_renderable_from_values(
         table.add_column("pedigree")
     if show_data:
         table.add_column("data")
-    if show_load_config:
-        table.add_column("load_config")
 
     for id, value in sorted(values.items(), key=lambda item: item[1].value_schema.type):
         row: List[RenderableType] = [id, value.value_schema.type, str(value.value_size)]
@@ -671,17 +808,17 @@ def create_renderable_from_values(
                 pedigree = value.pedigree.json(option=orjson.OPT_INDENT_2)
             row.append(pedigree)
         if show_data:
-            data = value._data_registry.render_data(
+            data = value._data_registry.pretty_print_data(
                 value_id=value.value_id, target_type="terminal_renderable", **config
             )
             row.append(data)
-        if show_load_config:
-            load_config = value.retrieve_load_config()
-            if load_config is None:
-                load_config_str: RenderableType = "-- not stored (yet) --"
-            else:
-                load_config_str = load_config.create_renderable()
-            row.append(load_config_str)
+        # if show_load_config:
+        #     load_config = value.retrieve_load_config()
+        #     if load_config is None:
+        #         load_config_str: RenderableType = "-- not stored (yet) --"
+        #     else:
+        #         load_config_str = load_config.create_renderable()
+        #     row.append(load_config_str)
         table.add_row(*row)
 
     return table
