@@ -13,27 +13,34 @@
     Read more about conftest.py under:
     https://pytest.org/latest/plugins.html
 """
-
 import pytest
 
 import os
 import tempfile
-import typing
 import uuid
+from pathlib import Path
 
-from kiara.config import KiaraConfig
-from kiara.data.onboarding.batch import BatchOnboard
-from kiara.kiara import Kiara
-from kiara.pipeline.config import PipelineConfig
+from kiara import Kiara
+from kiara.context.config import KiaraConfig
+from kiara.interfaces.python_api.batch import BatchOperation
 
 from .utils import INVALID_PIPELINES_FOLDER, MODULE_CONFIGS_FOLDER, PIPELINES_FOLDER
 
-TEMP_DIR = os.path.join(tempfile.gettempdir(), "kiara_tests")
 ROOT_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
 
 DATA_FOLDER = os.path.join(ROOT_DIR, "examples", "data")
 
 
+def create_temp_dir():
+    session_id = str(uuid.uuid4())
+    TEMP_DIR = Path(os.path.join(tempfile.gettempdir(), "kiara_tests"))
+
+    instance_path = os.path.join(
+        TEMP_DIR.resolve().absolute(), f"instance_{session_id}"
+    )
+    return instance_path
+
+
 @pytest.fixture
 def pipeline_paths():
 
@@ -76,49 +83,35 @@ def module_config_paths():
     return result
 
 
-@pytest.fixture
-def pipeline_configs(pipeline_paths) -> typing.Mapping[str, PipelineConfig]:
-
-    return {
-        name: PipelineConfig.parse_file(path) for name, path in pipeline_paths.items()
-    }
-
-
 @pytest.fixture
 def kiara() -> Kiara:
 
-    session_id = str(uuid.uuid4())
-
-    instance_data_store = os.path.join(TEMP_DIR, f"instance_{session_id}")
-    conf = KiaraConfig(data_store=instance_data_store)
-
-    kiara = Kiara(config=conf)
+    instance_path = create_temp_dir()
+    kc = KiaraConfig.create_in_folder(instance_path)
+    kiara = kc.create_context()
     return kiara
 
 
 @pytest.fixture(scope="module")
 def presseeded_data_store_minimal() -> Kiara:
 
-    session_id = str(uuid.uuid4())
-
-    instance_data_store = os.path.join(TEMP_DIR, f"instance_{session_id}")
-    conf = KiaraConfig(data_store=instance_data_store)
+    instance_path = create_temp_dir()
 
     pipeline_file = os.path.join(PIPELINES_FOLDER, "table_import.json")
 
-    kiara = Kiara(config=conf)
-    kiara.register_pipeline_description(pipeline_file, module_type_name="preseed")
-    results = kiara.run(
-        "preseed",
-        inputs={
-            "import_file__file_path": os.path.join(
-                ROOT_DIR, "examples", "data", "journals", "JournalNodes1902.csv"
-            )
-        },
-    )
+    kc = KiaraConfig.create_in_folder(instance_path)
 
-    table_value = results.get_value_obj("create_table_from_files__table")
-    table_value.save(aliases=["journal_nodes"])
+    kiara = kc.create_context()
+
+    batch_op = BatchOperation.from_file(pipeline_file, kiara=kiara)
+
+    inputs = {
+        "import_file__path": os.path.join(
+            ROOT_DIR, "examples", "data", "journals", "JournalNodes1902.csv"
+        )
+    }
+
+    batch_op.run(inputs=inputs, save="preseed_minimal")
 
     return kiara
 
@@ -126,13 +119,12 @@ def presseeded_data_store_minimal() -> Kiara:
 @pytest.fixture(scope="module")
 def preseeded_data_store() -> Kiara:
 
-    session_id = str(uuid.uuid4())
+    instance_path = create_temp_dir()
+    kc = KiaraConfig.create_in_folder(instance_path)
+    kiara = kc.create_context()
 
-    instance_data_store = os.path.join(TEMP_DIR, f"instance_{session_id}")
-    conf = KiaraConfig(data_store=instance_data_store)
-    kiara = Kiara(config=conf)
-
-    pipeline_folder = PIPELINES_FOLDER
+    pipeline = os.path.join(PIPELINES_FOLDER, "test_preseed_1.yaml")
+    batch_op = BatchOperation.from_file(pipeline, kiara=kiara)
 
     inputs = {
         "edges_file_path": os.path.join(DATA_FOLDER, "journals/JournalEdges1902.csv"),
@@ -142,24 +134,9 @@ def preseeded_data_store() -> Kiara:
         "city_column_name": "City",
         "label_column_name": "Label",
     }
-    pipeline = os.path.join(pipeline_folder, "test_preseed_1.yaml")
 
-    store_config_dict = {"outputs": [{"alias_template": "{{ field_name }}"}]}
+    batch_op.run(inputs=inputs, save="preseed")
 
-    onboard_config = {
-        "module_type": pipeline,
-        "inputs": inputs,
-        "store_config": store_config_dict,
-    }
-
-    # store_config = ValueStoreConfig(**store_config_dict)
-    onboarder = BatchOnboard.create(kiara=kiara, **onboard_config)
-    print(f"kiara data store: {kiara.data_store.base_path}")
-
-    results = onboarder.run("tests_1")
-    aliases = set()
-    for _a in results.values():
-        aliases.update(_a)
-    print(f"Onboarded example data, available aliases: {', '.join(aliases)}")
+    print(f"kiara data store: {kiara.data_registry.get_archive()}")
 
     return kiara
